import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import compliance_enabled_policies
from insights.parsers.compliance_enabled_policies import ComplianceEnabledPolicies
from insights.tests import context_wrap

COMPLIANCE_ENABLE_POLICIES = '''
{
    "enabled_policies": [
        {
            "id": "12345678-aaaa-bbbb-cccc-1234567890ab",
            "title": "advisor rule test - CIS Red Hat Enterprise Linux 8 Benchmark for Level 1 - Server",
            "description": "This profile defines a baseline that aligns to the Level 1 - Server",
            "business_objective": null,
            "compliance_threshold": 100.0,
            "total_system_count": 1,
            "type": "policy",
            "os_major_version": 8,
            "profile_title": "CIS Red Hat Enterprise Linux 8 Benchmark for Level 1 - Server",
            "ref_id": "xccdf_org.ssgproject.content_profile_cis_server_l1"
        },
        {
            "id": "12345678-aaaa-bbbb-cccc-1234567890xy",
            "title": "CIS Red Hat Enterprise Linux 8 Benchmark for Level 2 - Server",
            "description": "This profile defines a baseline that aligns to the Level 2 - Server",
            "business_objective": null,
            "compliance_threshold": 100.0,
            "total_system_count": 1,
            "type": "policy",
            "os_major_version": 8,
            "profile_title": "CIS Red Hat Enterprise Linux 8 Benchmark for Level 2 - Server",
            "ref_id": "xccdf_org.ssgproject.content_profile_cis"
        }
    ],
    "tailoring_policies": [
        {
            "ref_id": "xccdf_org.ssgproject.content_profile_cis_server_l1",
            "check_items": [
                {
                    "idref": "xccdf_org.ssgproject.content_rule_bios_disable_usb_boot",
                    "selected": "true"
                },
                {
                    "idref": "xccdf_org.ssgproject.content_rule_ssh_keys_passphrase_protected",
                    "selected": "true"
                },
                {
                    "idref": "xccdf_org.ssgproject.content_rule_sysctl_net_ipv4_conf_all_shared_media",
                    "selected": "true"
                },
                {
                    "idref": "xccdf_org.ssgproject.content_rule_accounts_password_pam_ocredit",
                    "selected": "false"
                }
            ]
        }
    ]
}
'''.strip()


def test_compliance_enabled_policies_skip():
    with pytest.raises(SkipComponent) as ex:
        ComplianceEnabledPolicies(context_wrap(""))
    assert "Empty output." in str(ex)


def test_compliance_enabled_policies():
    compliance_enabled_policies_info = ComplianceEnabledPolicies(context_wrap(COMPLIANCE_ENABLE_POLICIES))
    assert compliance_enabled_policies_info['enabled_policies'][0]['ref_id'] == 'xccdf_org.ssgproject.content_profile_cis_server_l1'
    assert compliance_enabled_policies_info['tailoring_policies'][0]['ref_id'] == 'xccdf_org.ssgproject.content_profile_cis_server_l1'
    assert len(compliance_enabled_policies_info['enabled_policies']) == 2
    assert len(compliance_enabled_policies_info['tailoring_policies'][0]['check_items']) == 4


def test_compliance_enabled_policies_only_enabled():
    """
    Test parser with only enabled_policies and no tailoring_policies.
    """
    only_enabled_policies = '''
    {
        "enabled_policies": [
            {
                "id": "12345678-aaaa-bbbb-cccc-1234567890ab",
                "ref_id": "xccdf_org.ssgproject.content_profile_standard"
            }
        ]
    }
    '''
    compliance_enabled_policies_info = ComplianceEnabledPolicies(context_wrap(only_enabled_policies))
    assert 'enabled_policies' in compliance_enabled_policies_info
    assert compliance_enabled_policies_info['enabled_policies'][0]['ref_id'] == 'xccdf_org.ssgproject.content_profile_standard'
    assert 'tailoring_policies' not in compliance_enabled_policies_info or not compliance_enabled_policies_info.get('tailoring_policies')


def test_compliance_enabled_policies_doc_examples():
    env = {
        "compliance_enabled_policies": ComplianceEnabledPolicies(context_wrap(COMPLIANCE_ENABLE_POLICIES)),
    }
    failed, total = doctest.testmod(compliance_enabled_policies, globs=env)
    assert failed == 0



from .. import Parser, parser, get_active_lines, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.rhn_hibernate_conf)
class RHNHibernateConf(LegacyItemAccess, Parser):

    def parse_content(self, content):
        """
        Parses rhn_hibernate.conf and returns a dict.
        - {
            "hibernate.c3p0.min_size": '5'
            "hibernate.c3p0.preferredTestQuery": "select 'c3p0 ping' from dual"
          }
        """
        hb_dict = {}
        for line in get_active_lines(content):
            if '=' in line:
                key, _, value = line.partition('=')
                hb_dict[key.strip()] = value.strip()
        self.data = hb_dict



import os
from tarfile import open as tar_open
import tarfile
import tempfile
import uuid
import insights.client.utilities as util
import insights.client.cert_auth
from insights.client.constants import InsightsConstants as constants
import re
try:
    from unittest import mock
    from unittest.mock import patch
except ImportError:
    import mock
    from mock.mock import patch
import six
import pytest
import errno
from json import loads as json_load


machine_id = str(uuid.uuid4())
remove_file_content = """
[remove]
commands=foo
files=bar
""".strip().encode("utf-8")


def test_display_name():
    assert util.determine_hostname(display_name='foo') == 'foo'


def test_determine_hostname():
    import socket
    hostname = socket.gethostname()
    fqdn = socket.getfqdn()
    assert util.determine_hostname() in (hostname, fqdn)
    assert util.determine_hostname() != 'foo'


def test_get_time():
    time_regex = re.match(r'\d{4}-\d{2}-\d{2}\D\d{2}:\d{2}:\d{2}\.\d+',
                          util.get_time())
    assert time_regex.group(0) is not None


def test_write_to_disk():
    content = 'boop'
    filename = '/tmp/testing'
    util.write_to_disk(filename, content=content)
    assert os.path.isfile(filename)
    with open(filename, 'r') as f:
        result = f.read()
    assert result == 'boop'
    util.write_to_disk(filename, delete=True) is None


def test_write_to_disk_with_broken_path():
    """
    Ensure that the `write_to_disk` method only
    executes without raising an exception if
    the encountered exception has error code 2
    (No such file or directory), otherwise
    raise the exception.
    """
    content = 'boop'
    filename = '/tmp/testing'
    util.write_to_disk(filename, content=content)
    assert os.path.exists(filename)

    with patch("os.remove") as mock_remove:
        mock_remove.side_effect = OSError()
        mock_remove.side_effect.errno = errno.ENOENT
        assert util.write_to_disk(filename, delete=True) is None

        mock_remove.side_effect.errno = errno.ENOTTY
        with pytest.raises(OSError):
            util.write_to_disk(filename, delete=True)
    os.remove(filename)


@patch("insights.client.utilities._get_rhsm_identity", lambda: None)
def test_generate_machine_id_with_no_subman():
    machine_id_regex = re.match(
        r'\w{8}-\w{4}-\w{4}-\w{4}-\w{12}',
        util.generate_machine_id(destination_file='/tmp/testmachineid')
    )
    assert machine_id_regex.group(0) is not None
    with open('/tmp/testmachineid', 'r') as _file:
        machine_id = _file.read()
    assert util.generate_machine_id(destination_file='/tmp/testmachineid') == machine_id
    os.remove('/tmp/testmachineid')


@pytest.mark.skipif(insights.client.cert_auth.RHSM_CONFIG is None, reason="RHSM config is not available")
@patch("insights.client.utilities.os.path.isfile", lambda _: True)
@patch("insights.client.cert_auth.rhsmCertificate")
@patch("insights.client.utilities.cert_auth.RHSM_CONFIG")
def test__get_rhsm_identity(mock_config, mock_rhsm_cert):
    mock_config.return_value = "non-None-value"

    cert = mock.MagicMock()
    cert.CERT = "cert.pem"
    cert.getConsumerId.return_value = machine_id
    mock_rhsm_cert.read.return_value = cert

    assert machine_id == util._get_rhsm_identity()


@patch("insights.client.utilities._get_rhsm_identity", lambda: machine_id)
def test_generate_machine_id_with_subman():
    """Use sub-man's identity certificate for generating new machine-id."""
    machine_id_file = "/tmp/test-machine-id"
    read_uuid = util.generate_machine_id(destination_file=machine_id_file)
    os.remove(machine_id_file)

    assert machine_id == read_uuid


def test_generate_machine_id_with_non_hyphen_id():
    content = '86f6f5fad8284730b708a2e44ba5c14a'
    filename = '/tmp/testmachineid'
    util.write_to_disk(filename, content=content)

    returned_uuid = str(uuid.UUID(content, version=4))

    assert util.generate_machine_id(destination_file=filename) == returned_uuid
    os.remove(filename)


def test_bad_machine_id():
    with mock.patch.object(util.sys, "exit") as mock_exit:
        with open('/tmp/testmachineid', 'w') as _file:
            _file.write("this_is_bad")
        util.generate_machine_id(destination_file='/tmp/testmachineid')
    assert mock_exit.call_args[0][0] == constants.sig_kill_bad
    os.remove('/tmp/testmachineid')


def test_expand_paths():
    assert util._expand_paths('/tmp') == ['/tmp']


def test_magic_plan_b():
    tf = tempfile.NamedTemporaryFile()
    with open(tf.name, 'w') as f:
        f.write('testing stuff')
    assert util.magic_plan_b(tf.name) == 'text/plain; charset=us-ascii'


def test_run_command_get_output():
    cmd = 'echo hello'
    assert util.run_command_get_output(cmd) == {'status': 0, 'output': u'hello\n'}


@patch('insights.client.utilities.wrapper_constants')
@patch.dict('insights.client.utilities.package_info', {'VERSION': '1', 'RELEASE': '1'})
def test_get_version_info_OK(wrapper_constants):
    '''
    insights_client constants are imported OK and version
    is reported. Return version as defined
    '''
    wrapper_constants.version = 1
    version_info = util.get_version_info()
    assert version_info == {'core_version': '1-1', 'client_version': 1}


@patch('insights.client.utilities.wrapper_constants', new=None)
@patch.dict('insights.client.utilities.package_info', {'VERSION': '1', 'RELEASE': '1'})
def test_get_version_info_no_module():
    '''
    insights_client constants cannot be imported,
    constants object is None. Return None version.
    '''
    version_info = util.get_version_info()
    assert version_info == {'core_version': '1-1', 'client_version': None}


@patch('insights.client.utilities.wrapper_constants')
@patch.dict('insights.client.utilities.package_info', {'VERSION': '1', 'RELEASE': '1'})
def test_get_version_info_no_version(wrapper_constants):
    '''
    insights_client constants are imported OK but
    constants object has no attribute "version."
    Return None version
    '''
    del wrapper_constants.version
    version_info = util.get_version_info()
    assert version_info == {'core_version': '1-1', 'client_version': None}


# TODO: DRY
@patch('insights.client.utilities.constants.registered_files',
       ['/tmp/insights-client.registered',
        '/tmp/redhat-access-insights.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       ['/tmp/insights-client.unregistered',
        '/tmp/redhat-access-insights.unregistered'])
def test_write_registered_file():
    util.write_registered_file()
    for r in constants.registered_files:
        assert os.path.isfile(r) is True
    for u in constants.unregistered_files:
        assert os.path.isfile(u) is False


@patch('insights.client.utilities.constants.registered_files',
       ['/tmp/insights-client.registered',
        '/tmp/redhat-access-insights.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       ['/tmp/insights-client.unregistered',
        '/tmp/redhat-access-insights.unregistered'])
def test_delete_registered_file():
    util.write_registered_file()
    util.delete_registered_file()
    for r in constants.registered_files:
        assert os.path.isfile(r) is False


@patch('insights.client.utilities.constants.registered_files',
       ['/tmp/insights-client.registered',
        '/tmp/redhat-access-insights.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       ['/tmp/insights-client.unregistered',
        '/tmp/redhat-access-insights.unregistered'])
def test_write_unregistered_file():
    util.write_unregistered_file()
    for r in constants.registered_files:
        assert os.path.isfile(r) is False
    for u in constants.unregistered_files:
        assert os.path.isfile(u) is True


@patch('insights.client.utilities.constants.registered_files',
       ['/tmp/insights-client.registered',
        '/tmp/redhat-access-insights.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       ['/tmp/insights-client.unregistered',
        '/tmp/redhat-access-insights.unregistered'])
def test_delete_unregistered_file():
    util.write_unregistered_file()
    util.delete_unregistered_file()
    for u in constants.unregistered_files:
        assert os.path.isfile(u) is False


def test_read_pidfile():
    '''
    Test a pidfile that exists
    '''
    if six.PY3:
        open_name = 'builtins.open'
    else:
        open_name = '__builtin__.open'

    with patch(open_name, create=True) as mock_open:
        mock_open.side_effect = [mock.mock_open(read_data='420').return_value]
        assert util.read_pidfile() == '420'


def test_read_pidfile_failure():
    '''
    Test a pidfile that does not exist
    '''
    if six.PY3:
        open_name = 'builtins.open'
    else:
        open_name = '__builtin__.open'

    with patch(open_name, create=True) as mock_open:
        mock_open.side_effect = IOError
        assert util.read_pidfile() is None


@patch('insights.client.utilities.threading.Thread')
@patch('insights.client.utilities.os.path.exists')
def test_systemd_notify_init_thread_no_socket(exists, thread):
    '''
    Test this function when NOTIFY_SOCKET is
    undefined, i.e. when we run the client on demand
    and not via systemd job
    '''
    exists.return_value = True
    util.systemd_notify_init_thread()
    thread.assert_not_called()


@patch('insights.client.utilities.Popen')
def test_systemd_notify(Popen):
    '''
    Test calling systemd-notify with a "valid" PID
    On RHEL 7, exists(/usr/bin/systemd-notify) == True
    '''
    Popen.return_value.communicate.return_value = ('', '')
    util._systemd_notify('420')
    Popen.assert_called_once()


@patch('insights.client.utilities.read_pidfile', mock.Mock(return_value=None))
@patch('insights.client.utilities.threading.Thread')
@patch('insights.client.utilities.os.path.exists')
@patch.dict('insights.client.utilities.os.environ', {'NOTIFY_SOCKET': '/tmp/test.sock'})
def test_systemd_notify_init_thread_failure_bad_pid(exists, thread):
    '''
    Test initializing systemd-notify loop with an invalid PID
    On RHEL 7, exists(/usr/bin/systemd-notify) == True
    '''
    exists.return_value = True
    util.systemd_notify_init_thread()
    exists.assert_not_called()
    thread.assert_not_called()


@patch('insights.client.utilities.threading.Thread')
@patch('insights.client.utilities.os.path.exists')
@patch.dict('insights.client.utilities.os.environ', {'NOTIFY_SOCKET': '/tmp/test.sock'})
def test_systemd_notify_init_thread_failure_rhel_6(exists, thread):
    '''
    Test calling systemd-notify on RHEL 6
    On RHEL 6, exists(/usr/bin/systemd-notify) == False
    '''
    exists.return_value = False
    util.systemd_notify_init_thread()
    thread.assert_not_called()


def test_get_tags():
    content = b"foo: bar"
    fp = tempfile.NamedTemporaryFile(delete=False)
    fp.write(content)
    fp.close()
    got = util.get_tags(fp.name)
    assert got == {"foo": "bar"}


def test_get_tags_empty():
    content = b""
    fp = tempfile.NamedTemporaryFile(delete=False)
    fp.write(content)
    fp.close()
    got = util.get_tags(fp.name)
    assert got == {}


def test_get_tags_nonexist():
    got = util.get_tags("/file/does/not/exist")
    assert got is None


def test_write_tags():
    tags = {'foo': 'bar'}
    fp = tempfile.NamedTemporaryFile()
    util.write_tags(tags, tags_file_path=fp.name)
    got = util.get_tags(fp.name)
    assert got == tags


@patch('insights.client.utilities.os.rename')
@patch('insights.client.utilities.os.path.exists')
def test_migrate_tags(path_exists, os_rename):
    '''
    Test the migrate_tags function for the following cases:
        1) tags.yaml does not exist, tags.conf does not exist
            - do nothing
        2) tags.yaml exists, tags.conf does not exist
            - do nothing
        3) tags.yaml does not exist, tags.conf exists
            - rename tags.conf to tags.yaml
        4) tags.yaml exists, tags.conf exists
            - do nothing
    '''
    # existence of tags.yaml is checked FIRST, tags.conf is checked SECOND
    #   mock side effects are according to this order

    # case 1
    path_exists.side_effect = [False, False]
    util.migrate_tags()
    os_rename.assert_not_called()
    os_rename.reset_mock()
    # case 2
    path_exists.side_effect = [True, False]
    util.migrate_tags()
    os_rename.assert_not_called()
    os_rename.reset_mock()
    # case 3
    path_exists.side_effect = [False, True]
    util.migrate_tags()
    os_rename.assert_called_once()
    os_rename.reset_mock()
    # case 4
    path_exists.side_effect = [True, True]
    util.migrate_tags()
    os_rename.assert_not_called()
    os_rename.reset_mock()


def mock_open(name, mode, files=[["meta_data/insights.spec-small", 1], ["meta_data/insights.spec-big", 1], ["data/insights/small", 1], ["data/insights/big", 100]]):
    base_path = "./insights-client"
    with tempfile.TemporaryFile(suffix='.tar.gz') as f:
        tarball = tar_open(fileobj=f, mode='w:gz')
        for file in files:
            member = tarfile.TarInfo(name=os.path.join(base_path, file[0]))
            member.size = file[1]
            tarball.addfile(member, None)
        return tarball


def mock_extract_file(self, filename):
    if "small" in filename:
        f = "small"
    else:
        f = "big"
    return f


def mock_json_load(filename):
    if "small" in filename:
        content = json_load('{"name": "insights.spec-small", "results": {"type": "insights.core.spec_factory.CommandOutputProvider", "object": { "relative_path": "insights/small"}}}')
    else:
        content = json_load('{"name": "insights.spec-big", "results": {"type": "insights.core.spec_factory.CommandOutputProvider", "object": { "relative_path": "insights/big"}}}')
    return content


# TODO: try to pass files as mock_open parameter
@patch('insights.client.utilities.tarfile.open', mock_open)
@patch('insights.client.utilities.json.load', mock_json_load)
@patch('insights.client.utilities.tarfile.TarFile.extractfile', mock_extract_file)
def test_largest_spec_in_archive():
    largest_file = util.largest_spec_in_archive("/tmp/insights-client.tar.gz")
    assert largest_file[0] == "insights/big"
    assert largest_file[1] == 100
    assert largest_file[2] == "insights.spec-big"


@pytest.mark.parametrize(
    "family,version,raw_os_release",
    [
        ("Red Hat Enterprise Linux", "9.5", ['NAME="Red Hat Enterprise Linux"\n', 'VERSION_ID="9.5"\n']),
        ("Red Hat Enterprise Linux", "8.9", ['NAME="Red Hat Enterprise Linux"\n', 'VERSION_ID="8.9"\n']),
        ("Red Hat Enterprise Linux", "7.9", ['NAME="Red Hat Enterprise Linux"\n', 'VERSION_ID="7.9"\n']),
        ("CentOS Stream", "10", ['NAME="CentOS Stream"\n', 'VERSION_ID="10"\n']),
        ("CentOS Stream", "8", ['NAME="CentOS Stream"\n', 'VERSION_ID="8"\n']),
        ("Fedora Linux", "41", ['NAME="Fedora Linux"\n', 'VERSION_ID=41\n']),
    ]
)
def test_os_release_info__os_release(family, version, raw_os_release):
    # type: (str, str, str) -> None
    with patch("insights.client.utilities._read_file", return_value=raw_os_release):
        actual_family, actual_version = util.os_release_info()
        assert (actual_family, actual_version) == (family, version)


@pytest.mark.parametrize(
    "family,version,raw_redhat_release",
    [
        ("Red Hat Enterprise Linux", "9.5", ["Red Hat Enterprise Linux release 9.5 (Plow)"]),
        ("Red Hat Enterprise Linux", "8.9", ["Red Hat Enterprise Linux release 8.9 (Ootpa)"]),
        ("Red Hat Enterprise Linux Server", "7.9", ["Red Hat Enterprise Linux Server release 7.9 (Maipo)"]),
        ("Red Hat Enterprise Linux Server", "6.10", ["Red Hat Enterprise Linux Server release 6.10 (Santiago)"]),
        ("Fedora", "41", ["Fedora release 41 (Forty One)"]),
        ("CentOS Stream", "10", ["CentOS Stream release 10"]),
        ("CentOS Stream", "9", ["CentOS Stream release 9"]),
        ("CentOS Stream", "8", ["CentOS Stream release 8"]),
        ("CentOS Linux", "8.4.2105", ["CentOS Linux release 8.4.2105"]),
        ("CentOS Linux", "7.9.2009", ["CentOS Linux release 7.9.2009 (Core)"]),
    ]
)
def test_os_release_info__redhat_release(family, version, raw_redhat_release):
    # type: (str, str, str) -> None
    with patch("insights.client.utilities._read_file", side_effect=[IOError, raw_redhat_release]):
        actual_family, actual_version = util.os_release_info()
        assert (actual_family, actual_version) == (family, version)


@pytest.mark.parametrize(
    "expected,distribution,version",
    [
        (9, "Red Hat Enterprise Linux", "9.5"),
        (8, "Red Hat Enterprise Linux", "8.9"),
        (7, "Red Hat Enterprise Linux Server", "7.9"),
        (6, "Red Hat Enterprise Linux Server", "6.10"),
        (10, "Fedora", "41"),
        (10, "CentOS Stream", "10"),
        (9, "CentOS Stream", "9"),
        (8, "CentOS Stream", "8"),
        (8, "CentOS Linux", "8.4.2105"),
        (7, "CentOS Linux", "7.9.2009"),
    ]
)
def test_get_rhel_version(expected, distribution, version):
    # type: (str, str, str) -> None
    with patch("insights.client.utilities.os_release_info", return_value=(distribution, version)):
        actual = insights.client.utilities.get_rhel_version()
        assert actual == expected


@pytest.mark.parametrize(
    "error,distribution,version",
    [
        # Defaults of utilities.os_release_info()
        ("Could not determine distribution family.", "Unknown", ""),
        # Unlikely irl; might happen with corrupted os-release (?)
        ("Could not determine version of 'Fedora'.", "Fedora", ""),
        # When running on non-RHEL distribution
        ("Unknown distribution 'Alpine Linux'.", "Alpine Linux", "3.21.2"),
    ]
)
def test_get_rhel_version_error(error, distribution, version):
    # type: (str, str, str) -> None
    with patch("insights.client.utilities.os_release_info", return_value=(distribution, version)):
        with pytest.raises(ValueError) as exc_info:
            insights.client.utilities.get_rhel_version()
        assert str(exc_info.value) == error



from pytest import mark

from insights.cleaner import Cleaner
from insights.client.config import InsightsConfig

test_data_sensitive = 'test \n\n\nabcd\n1234\npassword: p4ssw0rd\n'.splitlines()


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_password_line_changed_password(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {})
    ret = pp.clean_content(test_data_sensitive, [])
    # content IS changed
    assert test_data_sensitive != ret
    assert 'p4ssw0rd' not in ret[-1]
    assert '********' in ret[-1]


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_password_disabled_by_no_obfuscate(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {})
    ret = pp.clean_content(test_data_sensitive, ['password'])
    # content is NOT changed
    assert test_data_sensitive == ret


@mark.parametrize(
    ("line", "expected"),
    [
        ("password: p@ss_W0rd ?", "password: ******** ?"),
        ("password = p@ss_W0rd ?", "password = ******** ?"),
        ("password=p@ss_W0-d", "password=********"),
    ],
)
def test_clean_content_password(line, expected):
    c = InsightsConfig()
    pp = Cleaner(c, {'patterns': {'regex': ['myserver', r'my(\w*)key']}})
    actual = pp.clean_content(line)
    assert actual == expected



# -*- coding: utf-8 -*-
import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import subscription_manager
from insights.parsers.subscription_manager import (
    SubscriptionManagerID,
    SubscriptionManagerFacts,
    SubscriptionManagerStatus,
    SubscriptionManagerSyspurpose,
)
from insights.tests import context_wrap

FACTS_NORMAL_1 = """
aws_instance_id: 567890567890
network.ipv6_address: ::1
uname.sysname: Linux
uname.version: #1 SMP PREEMPT Fri Sep 2 16:07:40 EDT 2022
virt.host_type: rhev, kvm
virt.is_guest: True
""".strip()

FACTS_with_AB_LINES = """
You are attempting to use a locale that is not installed.
aws_instance_id: 567890567890
network.ipv6_address: ::1
uname.sysname: Linux
uname.version: #1 SMP PREEMPT Fri Sep 2 16:07:40 EDT 2022
virt.host_type: rhev, kvm
virt.is_guest: True
""".strip()

ID_NORMAL_1 = """
system identity: 6655c27c-f561-4c99-a23f-111111111110
name: rhel7.localdomain
org name: 1234567
org ID: 1234567
""".strip()

ID_with_AB_LINES = """
You are attempting to use a locale that is not installed.
system identity: 6655c27c-f561-4c99-a23f-111111111111
name: rhel7.localdomain
org name: 1234567
org ID: 1234567
""".strip()

ID_with_non_en_1 = """
identité du système : 6655c27c-f561-4c99-a23f-111111111112
nom : rhel7.localdomain
nom de l'organisation : 1234567
ID de l'organisation : 1234567
""".strip()

ID_with_non_en_2 = """
系统身份：6655c27c-f561-4c99-a23f-111111111113
名称：rhel7.localdomain
机构名称：1234567
机构 ID：1234567
""".strip()

ID_with_no_id = """
名称：rhel7.localdomain
机构名称：1234567
机构 ID：1234567
""".strip()

INPUT_NG_1 = """
XYC
Release not set
""".strip()

INPUT_NG_2 = ""


SUBSCRIPTION_MANAGER_STATUS = """
+-------------------------------------------+
   System Status Details
+-------------------------------------------+
Overall Status: Disabled
Content Access Mode is set to Simple Content Access. This host has access to content, regardless of subscription status.

System Purpose Status: Disabled
""".strip()

SUBSCRIPTION_MANAGER_STATUS_EMPTY = """

""".strip()

SUBSCRIPTION_MANAGER_STATUS_PRODUCT_KEY = """
+-------------------------------------------+
   System Status Details
+-------------------------------------------+
Overall Status: Insufficient

Red Hat Enterprise Linux for Virtual Datacenters, Standard:
- Guest has not been reported on any host and is using a temporary unmapped guest subscription. For more information, please see https://access.redhat.com/solutions/XXXX

System Purpose Status: Matched
""".strip()

SUBSCRIPTION_MANAGER_SYSPURPOSE = """
{
  "addons": [],
  "role": "Red Hat Enterprise Linux Server",
  "service_level_agreement": "Standard",
  "usage": "Development/Test"
}
""".strip()


def test_subman_facts():
    ret = SubscriptionManagerFacts(context_wrap(FACTS_NORMAL_1))
    for line in FACTS_NORMAL_1.splitlines():
        key, value = line.split(': ', 1)
        assert ret[key] == value

    ret = SubscriptionManagerFacts(context_wrap(FACTS_with_AB_LINES))
    for line in FACTS_NORMAL_1.splitlines():
        if ': ' not in line:
            continue
        key, value = line.split(': ', 1)
        assert ret[key] == value


def test_subman_id():
    ret = SubscriptionManagerID(context_wrap(ID_NORMAL_1))
    for line in ID_NORMAL_1.splitlines():
        key, value = line.split(': ', 1)
        assert ret[key] == value
    assert ret.identity == '6655c27c-f561-4c99-a23f-111111111110'

    ret = SubscriptionManagerID(context_wrap(ID_with_AB_LINES))
    for line in ID_with_AB_LINES.splitlines():
        if ': ' not in line:
            continue
        key, value = line.split(': ', 1)
        assert ret[key] == value
    assert ret.identity == '6655c27c-f561-4c99-a23f-111111111111'

    ret = SubscriptionManagerID(context_wrap(ID_with_non_en_1))
    for line in ID_with_non_en_1.splitlines():
        key, value = line.split(': ', 1)
        assert ret[key.strip()] == value
    assert ret.identity == '6655c27c-f561-4c99-a23f-111111111112'
    assert ret.uuid == '6655c27c-f561-4c99-a23f-111111111112'
    # Chinese with full-width colon
    ret = SubscriptionManagerID(context_wrap(ID_with_non_en_2))
    for line in ID_with_non_en_2.splitlines():
        key, value = line.split('：', 1)
        assert ret[key.strip()] == value
    assert ret.identity == '6655c27c-f561-4c99-a23f-111111111113'
    assert ret.uuid == '6655c27c-f561-4c99-a23f-111111111113'
    # No ID
    ret = SubscriptionManagerID(context_wrap(ID_with_no_id))
    assert ret.identity is None
    assert ret.uuid is None


def test_subman_status():
    ret = SubscriptionManagerStatus(context_wrap(SUBSCRIPTION_MANAGER_STATUS))
    assert ret['Overall Status'] == 'Disabled'
    assert ret['Content Access Mode'] == 'Simple Content Access'
    assert ret['System Purpose Status'] == 'Disabled'

    ret = SubscriptionManagerStatus(context_wrap(SUBSCRIPTION_MANAGER_STATUS_PRODUCT_KEY))
    assert ret['Overall Status'] == 'Insufficient'
    assert ret['Red Hat Enterprise Linux for Virtual Datacenters'] == 'Standard'
    assert ret['System Purpose Status'] == 'Matched'
    assert ret.unparsed_lines == [SUBSCRIPTION_MANAGER_STATUS_PRODUCT_KEY.splitlines()[-3]]


def test_subman_syspurpose():
    ret = SubscriptionManagerSyspurpose(context_wrap(SUBSCRIPTION_MANAGER_SYSPURPOSE))
    assert ret['addons'] == []
    assert ret.get('role') == 'Red Hat Enterprise Linux Server'
    assert ret['service_level_agreement'] == 'Standard'
    assert ret['usage'] == 'Development/Test'


def test_subman_facts_ng():
    with pytest.raises(SkipComponent):
        SubscriptionManagerFacts(context_wrap(INPUT_NG_1))

    with pytest.raises(SkipComponent):
        SubscriptionManagerFacts(context_wrap(INPUT_NG_2))


def test_subman_id_ng():
    with pytest.raises(SkipComponent):
        SubscriptionManagerID(context_wrap(INPUT_NG_1))

    with pytest.raises(SkipComponent):
        SubscriptionManagerID(context_wrap(INPUT_NG_2))


def test_subman_status_ng():
    with pytest.raises(SkipComponent):
        SubscriptionManagerStatus(context_wrap(SUBSCRIPTION_MANAGER_STATUS_EMPTY))


def test_doc_examples():
    env = {
        'rhsm_facts': SubscriptionManagerFacts(context_wrap(FACTS_with_AB_LINES)),
        'subman_id': SubscriptionManagerID(context_wrap(ID_with_AB_LINES)),
        'subman_status': SubscriptionManagerStatus(context_wrap(SUBSCRIPTION_MANAGER_STATUS)),
        'subman_syspurpose': SubscriptionManagerSyspurpose(
            context_wrap(SUBSCRIPTION_MANAGER_SYSPURPOSE)
        ),
    }
    failed, total = doctest.testmod(subscription_manager, globs=env)
    assert failed == 0



"""
RPM Version Comparison
======================
Compare the RPM version with `rpm.labelCompare` first, when the `rpm` module is
not available, try the local `_rpm_vercmp`.

The `_rpm_vercmp` is a nearly direct translation of rpm comparison code in the
rpm project at:
https://github.com/rpm-software-management/rpm/blob/master/lib/rpmvercmp.c

Handles all of the cases in the rpm test file, including the "buggy" tests
and non-ascii characters.

https://raw.githubusercontent.com/rpm-software-management/rpm/master/tests/rpmvercmp.at
"""

from collections import deque
from itertools import takewhile


def _rpm_vercmp(a, b):
    if a == b:
        return 0

    # ord check required for consistent unicode handling
    # empty string appended to simplify other checks below
    a = deque([c if ord(c) < 128 else "." for c in a])
    a.append("")

    b = deque([c if ord(c) < 128 else "." for c in b])
    b.append("")

    while a[0] or b[0]:
        for x in [a, b]:
            while x[0] and not x[0].isalnum() and x[0] not in "~^":
                x.popleft()

        # handle the tilde separator, it sorts before everything else
        if a[0] == "~" or b[0] == "~":
            if a[0] != "~":
                return 1
            if b[0] != "~":
                return -1
            a.popleft()
            b.popleft()
            continue

        # Handle caret separator. Concept is the same as tilde,
        # except that if one of the strings ends (base version),
        # the other is considered as higher version.
        if a[0] == "^" or b[0] == "^":
            if not a[0]:
                return -1
            if not b[0]:
                return 1
            if a[0] != "^":
                return 1
            if b[0] != "^":
                return -1
            a.popleft()
            b.popleft()
            continue

        # If we ran to the end of either, we are finished with the loop
        if not a[0] or not b[0]:
            break

        # grab first completely alpha or completely numeric segment
        if a[0].isdigit():
            l = deque(takewhile(lambda v: v.isdigit(), a))
            r = deque(takewhile(lambda v: v.isdigit(), b))
            isnum = True
        else:
            l = deque(takewhile(lambda v: v.isalpha(), a))
            r = deque(takewhile(lambda v: v.isalpha(), b))
            isnum = False

        # this cannot happen, as we previously tested to make sure that
        # the first string has a non-null segment
        if not l:
            return -1

        # take care of the case where the two version segments are
        # different types: one numeric, the other alpha (i.e. empty)
        # numeric segments are always newer than alpha segments
        # XXX See details from bugzilla #50977.
        if not r:
            return 1 if isnum else -1

        ll = len(l)
        lr = len(r)
        if isnum:
            # throw away any leading zeros - it's a number, right?
            for x in [l, r]:
                while x and x[0] == '0':
                    x.popleft()

            # whichever number has more digits wins
            lenl = len(l)
            lenr = len(r)
            if lenl > lenr:
                return 1

            if lenr > lenl:
                return -1

        if l > r:
            return 1

        if l < r:
            return -1

        for _ in range(ll):
            a.popleft()

        for _ in range(lr):
            b.popleft()

    # this catches the case where all numeric and alpha segments have
    # compared identically but the segment separating characters were
    # different
    if not a[0] and not b[0]:
        return 0

    # whichever version still has characters left over wins
    if not a[0]:
        return -1
    return 1


try:
    import rpm
    import six
    from functools import cmp_to_key

    def version_compare(left, right):
        # place the single string as 'version' in the middle of the tuple
        left = ('0', left, '0') if isinstance(left, six.string_types) else left
        right = ('0', right, '0') if isinstance(right, six.string_types) else right

        _l = cmp_to_key(rpm.labelCompare)(left)
        _r = cmp_to_key(rpm.labelCompare)(right)

        return -1 if _l < _r else 1 if _l > _r else 0

    def rpm_version_compare(left, right):
        if left is right:
            return 0
        return version_compare(
            (left.epoch, left.version, left.release), (right.epoch, right.version, right.release)
        )

except Exception:  # pragma: no cover
    version_compare = _rpm_vercmp

    def rpm_version_compare(left, right):
        for attr in ('epoch', 'version', 'release'):
            rc = _rpm_vercmp(getattr(left, attr, ''), getattr(right, attr, ''))
            if rc != 0:
                return rc
        return 0



import doctest

from insights.parsers import sos_conf
from insights.tests import context_wrap

CLIENT_CONF = """
[general]
batch = yes

[plugins]
disable = rpm, selinux, dovecot

[tunables]
""".strip()


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        sos_conf,
        globs={'sos_conf': sos_conf.SosConf(context_wrap(CLIENT_CONF))}
    )
    assert failed_count == 0


def test_sos_conf():
    conf = sos_conf.SosConf(context_wrap(CLIENT_CONF))
    assert conf is not None
    sections = list(conf.sections())
    assert len(sections) == 3
    assert 'tunables' in sections
    assert conf.has_option('general', 'batch')
    assert not conf.has_option('tunables', 'rpmva')
    assert conf.get('plugins', 'disable') == 'rpm, selinux, dovecot'
    assert conf.getboolean('general', 'batch')



"""
Pattern Redaction
=================
"""

import logging
import re

logger = logging.getLogger(__name__)


class Pattern(object):
    """
    Class for redacting "patterns" configured in "file-content.redaction.yaml".
    """

    def __init__(self, exclude, regex=False):
        self._exclude = exclude or []
        self._regex = regex

    def parse_line(self, line, **kwargs):
        # redact line per the file-content-redaction.yaml
        if not line:
            return line
        # patterns removal
        find = re.search if self._regex else lambda x, y: x in y
        if any(find(pat, line) for pat in self._exclude):
            logger.debug("Pattern matched, removing line: %s" % line.strip())
            # patterns found, remove it
            return None
        return line

    def generate_report(self, report_dir, archive_name):
        pass  # pragma: no cover



"""
Custom datasources relate to directory list
"""

import json
import os

from insights.core.context import HostContext
from insights.core.dr import get_name
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.blkid import BlockIDInfo
from insights.parsers.fstab import FSTab
from insights.parsers.lvm import Pvs
from insights.specs import Specs


def _list_items(spec):
    """Return a tuple of directories according to the spec filters."""
    filters = sorted(get_filters(spec))
    if filters:
        if len(filters) == 1 and 'R' not in get_name(spec).split('_')[-2]:
            """
            .. note::
                Insert a non-existing directory when there is only ONE target for
                the datasource to list, to make sure the directory be outputted.

                ==============================================================
                # list a single dir '/mnt'
                $ ls -lan /mnt
                total 0
                drwxr-xr-x.  2 0 0   6 Jun 21  2021 .
                dr-xr-xr-x. 17 0 0 224 Apr 17 16:45 ..
                --------------------------------------------------------------
                # list a single dir with this patch
                $ ls -lan /mnt _non_existing_
                ls: cannot access ' _non_existing_': No such file or directory
                /mnt:                      # <--<< the dir is outputted here
                total 0
                drwxr-xr-x.  2 0 0   6 Jun 21  2021 .
                dr-xr-xr-x. 17 0 0 224 Apr 17 16:45 ..
                ==============================================================
            """
            filters.append('_non_existing_')
        return filters
    raise SkipComponent


def _get_fstab_mounted_device_files(fstab_mounts, blkid_info):
    result = []
    blk_uuid_name_map = {}
    blk_label_name_map = {}
    for blk in blkid_info.data:
        uuid = blk.get("UUID", None)
        label = blk.get("LABEL", None)
        name = blk.get("NAME")
        if uuid:
            blk_uuid_name_map[uuid] = name
        if label:
            blk_label_name_map[label] = name
    for record in fstab_mounts:
        fs_spec = record['fs_spec']
        fs_spec_pair = fs_spec.split("=", 1)
        if fs_spec_pair[0] == "UUID" and fs_spec_pair[1] in blk_uuid_name_map:
            blkid_name = blk_uuid_name_map.get(fs_spec_pair[1])
            result.append(blkid_name)
        elif fs_spec_pair[0] == "LABEL" and fs_spec_pair[1] in blk_label_name_map:
            blkid_name = blk_label_name_map.get(fs_spec_pair[1])
            result.append(blkid_name)
        # Filter out devices like tmpfs, sysfs, proc ...
        elif "/" in fs_spec and "bind" not in record['fs_mntops']:
            result.append(fs_spec)
    return set(result)


@datasource(HostContext)
def list_with_la(broker):
    return ' '.join(_list_items(Specs.ls_la_dirs))


@datasource(HostContext)
def list_with_la_filtered(broker):
    return ' '.join(_list_items(Specs.ls_la_filtered_dirs))


@datasource(HostContext, optional=[FSTab])
def list_with_lan(broker):
    filters = set(_list_items(Specs.ls_lan_dirs))
    if 'fstab_mounted.dirs' in filters and FSTab in broker:
        filters.remove('fstab_mounted.dirs')
        for mntp in broker[FSTab].mounted_on.keys():
            mnt_point = os.path.dirname(mntp)
            filters.add(mnt_point) if mnt_point else None
    return ' '.join(sorted(filters))


@datasource(HostContext)
def list_with_lan_filtered(broker):
    return ' '.join(_list_items(Specs.ls_lan_filtered_dirs))


@datasource(HostContext)
def list_with_lanL(broker):
    return ' '.join(_list_items(Specs.ls_lanL_dirs))


@datasource(HostContext)
def list_with_lanR(broker):
    return ' '.join(_list_items(Specs.ls_lanR_dirs))


@datasource(HostContext)
def list_with_lanRL(broker):
    return ' '.join(_list_items(Specs.ls_lanRL_dirs))


@datasource(HostContext)
def list_with_laRZ(broker):
    return ' '.join(_list_items(Specs.ls_laRZ_dirs))


@datasource(HostContext)
def list_with_laZ(broker):
    return ' '.join(_list_items(Specs.ls_laZ_dirs))


@datasource(HostContext, optional=[FSTab, BlockIDInfo, Pvs])
def list_files_with_lH(broker):
    filters = set(_list_items(Specs.ls_lH_files))
    files = set(_f for _f in filters if not os.path.isdir(_f))
    if 'fstab_mounted.devices' in filters and FSTab in broker and BlockIDInfo in broker:
        files.remove('fstab_mounted.devices')
        fstab_mounts = broker[FSTab]
        blkid_info = broker[BlockIDInfo]
        files.update(_get_fstab_mounted_device_files(fstab_mounts, blkid_info))
    if 'pvs.devices' in filters and Pvs in broker:
        files.remove('pvs.devices')
        pvs_info = broker[Pvs]
        files.update(set([item['PV'] for item in pvs_info]))
    if files:
        return ' '.join(sorted(files))
    raise SkipComponent


@datasource(HostContext)
def files_dirs_number(broker):
    """Return a dict of file numbers from the spec filter"""
    result = {}
    for item in sorted(get_filters(Specs.files_dirs_number_filter)):
        item = os.path.join(item, '')  # ensure endswith "/", --> directory
        if os.path.exists(item):
            result[item] = dict(files_number=0, dirs_number=0)
            for name in os.listdir(item):
                path = os.path.join(item, name)
                result[item]['files_number'] += 1 if os.path.isfile(path) else 0
                result[item]['dirs_number'] += 1 if os.path.isdir(path) else 0
    if result:
        return DatasourceProvider(
            content=json.dumps(result, sort_keys=True),
            relative_path='insights_datasources/files_dirs_number',
        )
    raise SkipComponent



from insights.parsers import hammer_task_list
from insights.tests import context_wrap

import doctest

hammer_task_list_csv_data = '''
ID,Name,Owner,Started at,Ended at,State,Result,Task action,Task errors
92b732ea-7423-4644-8890-80e054f1799a,,foreman_api_admin,2016/11/11 07:18:32,2016/11/11 07:18:34,stopped,success,Refresh repository,""
e9cb6455-a433-467e-8404-7d01bd726689,,foreman_api_admin,2016/11/11 07:18:28,2016/11/11 07:18:31,stopped,success,Refresh repository,""
e30f3e7e-c023-4380-9594-337fdc4967e4,,foreman_api_admin,2016/11/11 07:18:24,2016/11/11 07:18:28,stopped,success,Refresh repository,""
3197f6a1-891f-4f42-9e4d-92c83c3ed035,,foreman_api_admin,2016/11/11 07:18:20,2016/11/11 07:18:24,stopped,success,Refresh repository,""
22169621-7175-411c-86be-46b4254a4e77,,foreman_api_admin,2016/11/11 07:18:16,2016/11/11 07:18:19,stopped,success,Refresh repository,""
f111e8f7-c956-470b-abb6-2e436ecd5866,,foreman_api_admin,2016/11/11 07:18:14,2016/11/11 07:18:16,stopped,success,Refresh repository,""
dfc702ea-ce46-427c-8a07-43e2a68e1320,,foreman_api_admin,2016/11/11 07:18:12,2016/11/11 07:18:14,stopped,success,Refresh repository,""
e8cac892-e666-4f2c-ab97-2be298da337e,,foreman_api_admin,2016/11/11 07:18:09,2016/11/11 07:18:12,stopped,success,Refresh repository,""
e6c1e1b2-a29d-4fd0-891e-e736dc9b7150,,,2016/11/11 07:14:06,2016/11/12 05:10:17,stopped,success,Listen on candlepin events,""
44a42c49-3038-4cae-8067-4d1cc305db05,,,2016/11/11 07:11:44,2016/11/11 07:12:47,stopped,success,Listen on candlepin events,""
72669288-54ac-41ba-a3b2-314a2c81f438,,,2016/11/11 06:57:15,2016/11/11 07:07:03,stopped,success,Listen on candlepin events,""
1314c91e-19d6-4d71-9bca-31db0df0aad2,,foreman_admin,2016/11/11 06:55:59,2016/11/11 06:55:59,stopped,error,Update for host sat62disc.example.org,"There was an issue with the backend service candlepin: 404 Resource Not Found, There was an issue with the backend service candlepin: 404 Resource Not Found"
303ef924-9845-4267-a705-194a4ebfbcfb,,foreman_admin,2016/11/11 06:55:58,2016/11/11 06:55:58,stopped,error,Package Profile Update,500 Internal Server Error
cffa5990-23ba-49f5-828b-ae0c77e8257a,,foreman_admin,2016/11/11 06:55:53,2016/11/11 06:55:56,stopped,error,Update for host sat62disc.example.org,"There was an issue with the backend service candlepin: 404 Resource Not Found, There was an issue with the backend service candlepin: 404 Resource Not Found"
07780e8f-dd81-49c4-a792-c4d4d162eb10,,foreman_admin,2016/11/11 06:55:50,2016/11/11 06:55:51,stopped,error,Update for host sat62disc.example.org,"There was an issue with the backend service candlepin: 404 Resource Not Found, There was an issue with the backend service candlepin: 404 Resource Not Found"
749a17a1-a8cb-46f0-98f6-017576481df8,,foreman_admin,2016/11/11 06:51:28,2016/11/11 06:51:29,stopped,error,Update for host sat62disc.example.org,"There was an issue with the backend service candlepin: 404 Resource Not Found, There was an issue with the backend service candlepin: 404 Resource Not Found"
d8f41819-b492-46e5-b0e3-ead3b4b6810c,,foreman_admin,2016/11/11 06:51:22,2016/11/11 06:51:28,stopped,error,Package Profile Update,500 Internal Server Error
'''

hammer_running_task_on_sat_65 = """
ID,Action,State,Result,Started at,Ended at,Owner,Task errors
4a22f7c6-ca65-4aee-8292-209e5742cce8,Listen on candlepin events ,running,pending,2019/09/10 05:46:39,"",,""
f3faa058-f155-4200-a70b-279b85772ca6,Monitor Event Queue ,running,pending,2019/09/10 05:46:39,"",,""
""".strip()

hammer_running_task_on_sat_64 = """
ID,Name,Owner,Started at,Ended at,State,Result,Task action,Task errors
2ffc3f0d-da4f-4b05-b86a-443ff21d27d3,"Monitor Event Queue {""locale""=>""en"", ""current_user_id""=>nil}",,2019/09/02 08:38:37,"",running,pending,Monitor Event Queue,""
32e6c73f-d373-4e40-8c11-4674bf346cbc,"Listen on candlepin events {""locale""=>""en"", ""current_user_id""=>nil}",,2019/09/02 08:38:36,"",running,pending,Listen on candlepin events,""
""".strip()

hammer_all_tasks_list_on_sat_64 = """
ID,Name,Owner,Started at,Ended at,State,Result,Task action,Task errors
80645ff3-b168-4de9-8751-536ce2f319a9,"Remove orphans {""services_checked""=>[""pulp"", ""pulp_auth""],
 ""capsule_id""=>1,
 ""remote_user""=>""admin"",
 ""remote_cp_user""=>""admin""}",foreman_admin,2019/08/05 02:00:31,2019/08/05 02:00:33,running,success,Remove orphans,""
c39c3dca-4dca-45d3-8d0f-1f1307ba174c,Create RSS notifications,,2019/08/04 14:59:46,2019/08/04 14:59:47,stopped,success,Create RSS notifications,""
09c8e998-b674-42f5-a12b-68b8662aa261,Pulp disk space notification,,2019/08/04 14:59:30,2019/08/04 14:59:30,stopped,success,Pulp disk space notification,""
8574e54f-1329-4f08-89e3-f2427a3d4318,Scan cdn,admin,2019/08/23 09:38:15,2019/08/23 09:38:17,stopped,success,Scan cdn,""
436acc7d-35ea-4f02-a389-b6f359e106b6,Update organization 'Default Organization',admin,2019/08/23 09:37:06,2019/08/23 09:37:06,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/f3f1704c-679e-4c49-a9e7-e896fa7bb9fd"
ac350b3f-6128-4ab9-989f-c7546fe54cb6,Update organization 'Default Organization',admin,2019/08/23 09:37:06,2019/08/23 09:37:07,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/f3f1704c-679e-4c49-a9e7-e896fa7bb9fd"
51670134-7363-48cc-8db8-eb42f97763a5,Update organization 'Default Organization',admin,2019/08/23 09:37:05,2019/08/23 09:37:05,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/f3f1704c-679e-4c49-a9e7-e896fa7bb9fd"
cbe81691-7ddf-4857-a112-2a73a2041f8a,Update organization 'Default Organization',admin,2019/08/23 09:37:04,2019/08/23 09:37:04,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/f3f1704c-679e-4c49-a9e7-e896fa7bb9fd"
dc472eb1-f18b-483f-a77d-1953a3c0aa58,Update organization 'Default Organization',admin,2019/08/23 09:37:02,2019/08/23 09:37:02,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/f3f1704c-679e-4c49-a9e7-e896fa7bb9fd"
f3f1704c-679e-4c49-a9e7-e896fa7bb9fd,Update organization 'Default Organization',admin,2019/08/23 09:31:46,2019/08/23 09:38:35,stopped,success,Update,""
8e1aaaca-73a2-46e8-b792-d7e4af21e3bf,Update organization 'Default Organization',admin,2019/08/23 09:30:36,2019/08/23 09:30:36,stopped,error,Update,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/1430bd27-dacd-4a91-bb1f-9930550a3879, Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: read
Conflicts with tasks:
- https://vm37-76.gsslab.pek2.redhat.com/foreman_tasks/tasks/1430bd27-dacd-4a91-bb1f-9930550a3879"
""".strip()

hammer_all_tasks_list_on_sat_65 = '''
ID,Action,State,Result,Started at,Ended at,Owner,Task errors
1d9fb216-ce00-446d-b001-97b38cb02c9c,Update for host dhcp-136-65.nay.redhat.com ,stopped,error,2019/06/13 05:27:10,2019/06/13 05:27:10,foreman_admin,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: update
Conflicts with tasks:
- https://vm37-45.gsslab.pek2.redhat.com/foreman_tasks/tasks/9dc0ef6d-08a6-41d0-b941-5f0e79da0428"
9dc0ef6d-08a6-41d0-b941-5f0e79da0428,Update for host dhcp-136-65.nay.redhat.com ,stopped,success,2019/06/13 05:27:07,2019/06/13 05:27:12,foreman_admin,""
f38a6d82-b85b-48b8-85f8-44030f37161f,Listen on candlepin events ,stopped,error,2019/06/13 03:01:38,2019/06/13 03:01:38,,Action Actions::Candlepin::ListenOnCandlepinEvents is already active
f0ed64ad-15e4-4710-877e-ab972e7c3471,Listen on candlepin events ,stopped,error,2019/06/13 02:27:02,2019/06/13 02:27:02,,Action Actions::Candlepin::ListenOnCandlepinEvents is already active
0d229121-0da4-4d36-9373-30222c300bdf,Update for host dhcp-136-65.nay.redhat.com ,running,success,2019/06/12 17:27:06,2019/06/12 17:27:07,foreman_admin,""
4a816bcf-c035-400c-86db-161d5287b2a5,Update for host dhcp-136-65.nay.redhat.com ,stopped,error,2019/06/12 13:27:07,2019/06/12 13:27:07,foreman_admin,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: update
Conflicts with tasks:
- https://vm37-45.gsslab.pek2.redhat.com/foreman_tasks/tasks/10e70826-acff-4ff9-94c3-174c3b289745"
10e70826-acff-4ff9-94c3-174c3b289745,Update for host dhcp-136-65.nay.redhat.com ,stopped,success,2019/06/12 13:27:06,2019/06/12 13:27:08,foreman_admin,""
3348390f-db34-448b-8775-e22f12a41dd4,Update for host vm37-186.gsslab.pek2.redhat.com ,stopped,success,2019/06/12 11:42:50,2019/06/12 11:42:51,foreman_admin,""
79e37493-45d1-490d-8164-2847e1677675,Update for host dhcp-136-65.nay.redhat.com ,running,error,2019/06/12 09:27:03,2019/06/12 09:27:03,foreman_admin,"Required lock is already taken by other running tasks.
Please inspect their state, fix their errors and resume them.

Required lock: update
Conflicts with tasks:
- https://vm37-45.gsslab.pek2.redhat.com/foreman_tasks/tasks/5c46cdec-ad59-4bb1-bec7-61ed1aa1b504"
87e5a505-6bc6-41ce-b56e-429f756329a3,"Create activation_key {""text""=>""activation key 'act-keys-1'"", ""link""=>""/activation_keys/1/info""} organization {""text""=>""organization 'org1'"", ""link""=>""/organizations/3/edit""}",stopped,success,2019/05/23 09:11:37,2019/05/23 09:11:38,admin,""
'''.strip()


def test_HTL_doc_examples():
    tasks = hammer_task_list.HammerTaskList(context_wrap(hammer_task_list_csv_data))
    globs = {
        'tasks': tasks
    }
    failed, tested = doctest.testmod(hammer_task_list, globs=globs)
    assert failed == 0


def test_tasks_on_sat_65():
    tasks = hammer_task_list.HammerTaskList(context_wrap(hammer_running_task_on_sat_65))
    assert len(tasks.running_tasks) == 2
    assert tasks[0]['Action'] == 'Listen on candlepin events'


def test_tasks_on_sat_64():
    tasks = hammer_task_list.HammerTaskList(context_wrap(hammer_running_task_on_sat_64))
    assert len(tasks.running_tasks) == 2
    assert tasks[0]['Task action'] == 'Monitor Event Queue'


def test_all_tasks_on_sat_64():
    tasks = hammer_task_list.HammerTaskList(context_wrap(hammer_all_tasks_list_on_sat_64))
    assert len(tasks) == 11
    assert len(tasks.running_tasks) == 1


def test_all_tasks_on_sat_65():
    tasks = hammer_task_list.HammerTaskList(context_wrap(hammer_all_tasks_list_on_sat_65))
    assert len(tasks) == 10
    assert len(tasks.running_tasks) == 2



"""
UpdateInfo - command ``yum updateinfo list -C``
===============================================
Provides a list of available advisories
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import parse_delimited_table
from insights.specs import Specs


@parser(Specs.yum_updateinfo)
class YumUpdateinfo(CommandParser):
    """
    Class for parsing the output of `yum updateinfo list -C`.

    Expected output of the command is::

        RSHA-2020-0001 security       firefox-83.0-13.fc32.x86_64
        RHBA-2020-0002 bugfix         flatpak-libs-1.8.3-1.fc32.x86_64
        RHEA-2020-0003 enhancement    flatpak-selinux-1.8.3-1.fc32.noarch

    Examples:
        >>> len(updateinfo.items)
        3
        >>> updateinfo.items[0]
        {'advisory': 'RSHA-2020-0001', 'type': 'security', 'package': 'firefox-83.0-13.fc32.x86_64'}

    """

    def parse_content(self, content):
        """Parse the command output"""
        with_header = ['advisory type package'] + content
        table = parse_delimited_table(with_header)
        if not table:
            raise SkipComponent('No data.')
        self._items = table

    @property
    def items(self):
        """
        list: Updatable packages, along with minimal advisory metadata
        """
        return self._items



import pytest
from mock.mock import patch, mock_open
from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.env import ld_library_path_global_conf

ENV_FILE_1 = """
LD_LIBRARY_PATH="/path/to/test"
export LD_LIBRARY_PATH
""".strip()

ENV_FILE_2 = """
export  LD_LIBRARY_PATH="/path/to/test"
""".strip()

ENV_FILE_3 = """
export LD_LIBRARY_PATH="/path/to/test" TEST="test"
""".strip()

ENV_CONFIG = {
    "/etc/environment": {"content": ENV_FILE_1, "isfile": True, "isdir": False},
    "/etc/env.d": {"content": "# export LD_LIBRARY_PATH", "isfile": False, "isdir": True},
    "/etc/env.d/test.conf": {"content": ENV_FILE_2, "isfile": True, "isdir": False},
    "/etc/profile": {
        "content": "export TEST=/test\n unset LD_LIBRARY_PATH",
        "isfile": True,
        "isdir": False,
    },
    "/etc/profile.d": {"content": "export TEST # LD_LIBRARY_PATH", "isfile": False, "isdir": True},
    "/etc/profile.d/test.conf": {"content": "", "isfile": True, "isdir": False},
    "/etc/bashrc": {"content": "", "isfile": True, "isdir": False},
    "/etc/bash.bashrc": {"content": "", "isfile": True, "isdir": False},
    "/root/.bash_profile": {"content": ENV_FILE_3, "isfile": True, "isdir": False},
    "/root/.bashrc": {"content": "", "isfile": True, "isdir": False},
    "/root/.profile": {"content": "LD_LIBRARY_PATH=/path/to/test", "isfile": True, "isdir": False},
    "/root/.cshrc": {"content": "", "isfile": True, "isdir": False},
    "/root/.zshrc": {"content": "", "isfile": True, "isdir": False},
    "/root/.tcshrc": {"content": "", "isfile": True, "isdir": False},
}

RELATIVE_PATH = 'insights_datasources/ld_library_path_global_conf'

EXPECTED_RESULT = """
{"export_files": ["/etc/environment", "/etc/env.d/test.conf", "/root/.bash_profile"], "unset_files": ["/etc/profile"]}
""".strip()


def _open_side_effect(name, mode):
    return mock_open(read_data=ENV_CONFIG.get(name, {}).get("content", ""))()


def _isfile_side_effect(name):
    return ENV_CONFIG.get(name, {}).get("isfile", False)


def _isdir_return_value(name):
    return ENV_CONFIG.get(name, {}).get("isdir", False)


@patch('insights.specs.datasources.env.open', side_effect=_open_side_effect)
@patch('os.path.exists', return_value=True)
@patch('os.path.isfile', side_effect=_isfile_side_effect)
@patch('os.path.isdir', return_value=_isdir_return_value)
@patch('os.listdir')
def test_ld_library_path_global_conf(
    mock_listdir, mock_isdir, mock_isfile, mock_exists, mock_ds_open
):
    mock_listdir.return_value = ["test.conf"]

    broker = {}
    result = ld_library_path_global_conf(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=EXPECTED_RESULT, relative_path=RELATIVE_PATH)
    assert result.relative_path == expected.relative_path
    assert result.content == expected.content


@patch('os.path.exists', return_value=False)
def test_ld_library_path_global_conf_without_env_files(mock_exists):
    broker = {}
    with pytest.raises(SkipComponent) as e:
        ld_library_path_global_conf(broker)
    assert 'SkipComponent' in str(e)


@patch('insights.specs.datasources.env.open', read_data="")
@patch('os.path.exists', return_value=True)
@patch('os.path.isfile', side_effect=_isfile_side_effect)
@patch('os.path.isdir', return_value=_isdir_return_value)
@patch('os.listdir')
def test_ld_library_path_global_conf_all_env_files_are_empty(
    mock_listdir, mock_isdir, mock_isfile, mock_exists, mock_ds_open
):
    mock_listdir.return_value = ["test.conf"]

    broker = {}
    with pytest.raises(SkipComponent) as e:
        ld_library_path_global_conf(broker)
    assert 'SkipComponent' in str(e)



"""
Auto Configuration Helper
"""
from __future__ import absolute_import
import logging
import requests
import re

from .constants import InsightsConstants as constants
from .cert_auth import rhsmCertificate
from .connection import InsightsConnection

logger = logging.getLogger(__name__)
APP_NAME = constants.app_name


def _is_rhn_or_rhsm(hostname):
    return (hostname == 'subscription.rhn.redhat.com' or
            hostname == 'subscription.rhsm.redhat.com')


def _is_staging_rhsm(hostname):
    return hostname == 'subscription.rhsm.stage.redhat.com'


def verify_connectivity(config):
    """
    Verify connectivity to satellite server
    """
    logger.debug("Verifying Connectivity")
    ic = InsightsConnection(config)
    try:
        branch_info = ic.get_branch_info()
    except requests.ConnectionError as e:
        logger.debug(e)
        logger.debug("Failed to connect to satellite")
        return False
    except LookupError as e:
        logger.debug(e)
        logger.debug("Failed to parse response from satellite")
        return False

    try:
        remote_leaf = branch_info['remote_leaf']
        return remote_leaf
    except LookupError as e:
        logger.debug(e)
        logger.debug("Failed to find accurate branch_info")
        return False


def set_auto_configuration(config, hostname, ca_cert, proxy, is_satellite, is_stage, rhsm_no_proxy=None):
    """
    Set config based on discovered data
    """
    logger.debug("Attempting to auto configure!")
    logger.debug("Attempting to auto configure hostname: %s", hostname)
    logger.debug("Attempting to auto configure CA cert: %s", ca_cert)
    logger.debug("Attempting to auto configure proxy: %s", proxy)
    logger.debug("Attempting to auto configure no_proxy: %s", rhsm_no_proxy)
    saved_base_url = config.base_url
    if ca_cert is not None:
        saved_cert_verify = config.cert_verify
        config.cert_verify = ca_cert
    if proxy is not None:
        saved_proxy = config.proxy
        config.proxy = proxy
    if rhsm_no_proxy and rhsm_no_proxy != '':
        config.no_proxy = rhsm_no_proxy
    if is_satellite:
        # satellite
        config.base_url = hostname + '/r/insights'
        logger.debug('Auto-configured base_url: %s', config.base_url)
    else:
        # connected directly to RHSM
        if is_stage:
            config.base_url = hostname + '/api'
        else:
            config.base_url = hostname + '/r/insights'
        logger.debug('Auto-configured base_url: %s', config.base_url)
        logger.debug('Not connected to Satellite, skipping branch_info')
        # direct connection to RHSM, skip verify_connectivity
        return

    if not verify_connectivity(config):
        logger.warn("Could not auto configure, falling back to static config")
        logger.warn("See %s for additional information",
                    constants.default_log_file)
        config.base_url = saved_base_url
        if proxy is not None:
            if saved_proxy is not None and saved_proxy.lower() == 'none':
                saved_proxy = None
            config.proxy = saved_proxy
        if ca_cert is not None:
            config.cert_verify = saved_cert_verify


def _importInitConfig():
    from rhsm.config import initConfig
    return initConfig()


def _try_satellite6_configuration(config):
    """
    Try to autoconfigure for Satellite 6
    """
    try:
        rhsm_config = _importInitConfig()

        logger.debug('Trying to autoconfigure...')
        cert = open(rhsmCertificate.certpath(), 'r').read()
        key = open(rhsmCertificate.keypath(), 'r').read()
        rhsm = rhsmCertificate(key, cert)
        is_satellite = False
        is_stage = False

        # This will throw an exception if we are not registered
        logger.debug('Checking if system is subscription-manager registered')
        rhsm.getConsumerId()
        logger.debug('System is subscription-manager registered')

        rhsm_hostname = rhsm_config.get('server', 'hostname')
        rhsm_hostport = rhsm_config.get('server', 'port')
        rhsm_proxy_hostname = rhsm_config.get('server', 'proxy_hostname').strip()
        rhsm_proxy_port = rhsm_config.get('server', 'proxy_port').strip()
        rhsm_proxy_user = rhsm_config.get('server', 'proxy_user').strip()
        rhsm_proxy_pass = rhsm_config.get('server', 'proxy_password').strip()
        rhsm_no_proxy = rhsm_config.get('server', 'no_proxy').strip()
        if rhsm_no_proxy.lower() == 'none' or rhsm_no_proxy == '':
            rhsm_no_proxy = None

        proxy = None

        if rhsm_proxy_hostname != "":
            logger.debug("Found rhsm_proxy_hostname %s", rhsm_proxy_hostname)
            proxy = "http://"
            if rhsm_proxy_user != "" and rhsm_proxy_pass != "":
                logger.debug("Found user and password for rhsm_proxy")
                proxy = proxy + rhsm_proxy_user + ":" + rhsm_proxy_pass + "@"
            proxy = proxy + rhsm_proxy_hostname + ':' + rhsm_proxy_port
            logger.debug("RHSM Proxy: %s", proxy)
        logger.debug("Found %sHost: %s, Port: %s",
                     ('' if _is_rhn_or_rhsm(rhsm_hostname) or
                         _is_staging_rhsm(rhsm_hostname)
                         else 'Satellite 6 Server '),
                     rhsm_hostname, rhsm_hostport)
        rhsm_ca = rhsm_config.get('rhsm', 'repo_ca_cert')
        logger.debug("Found CA: %s", rhsm_ca)
        logger.debug("Setting authmethod to CERT")
        config.authmethod = 'CERT'

        # Directly connected to Red Hat, use cert auth directly with the api
        if _is_rhn_or_rhsm(rhsm_hostname):
            # URL changes. my favorite
            logger.debug("Connected to Red Hat Directly, using cert-api")
            rhsm_hostname = 'cert-api.access.redhat.com'
            rhsm_ca = None
        elif _is_staging_rhsm(rhsm_hostname):
            logger.debug('Connected to staging RHSM, using cert.cloud.stage.redhat.com')
            rhsm_hostname = 'cert.cloud.stage.redhat.com'
            # never use legacy upload for staging
            config.legacy_upload = False
            config.cert_verify = True
            is_stage = True
            rhsm_ca = None
        else:
            # Set the host path
            # 'rhsm_hostname' should really be named ~ 'rhsm_host_base_url'
            rhsm_hostname = rhsm_hostname + ':' + rhsm_hostport + '/redhat_access'
            is_satellite = True

        logger.debug("Trying to set auto_configuration")
        set_auto_configuration(config, rhsm_hostname, rhsm_ca, proxy, is_satellite, is_stage, rhsm_no_proxy=rhsm_no_proxy)
        return True
    except Exception as e:
        logger.debug(e)
        logger.debug('System is NOT subscription-manager registered')
        return False


def try_auto_configuration(config):
    """
    Try to auto-configure if we are attached to a sat6
    """
    if config.auto_config and not config.offline:
        _try_satellite6_configuration(config)
    if not config.legacy_upload and re.match(r'(.+)?\/r\/insights', config.base_url):
        # When to append /platform
        #   base url ~= console.redhat.com/r/insights
        #   base url ~= cert-api.access.redhat.com/r/insights
        #   base url ~= satellite.host.example.com/redhat_access/r/insights
        # When not to append /platform
        #   base url ~= console.redhat.com/api
        config.base_url = config.base_url + '/platform'
    logger.debug('Updated base_url: %s', config.base_url)



from insights.parsers.facter import Facter
from insights.tests import context_wrap

FACTS = """
COMMAND> facter

architecture => x86_64
augeasversion => 1.1.0
bios_release_date => 04/14/2014
bios_vendor => Phoenix Technologies LTD
bios_version => 6.00
blockdevice_fd0_size => 4096
blockdevice_sda_model => Virtual disk
blockdevice_sda_size => 53687091200
blockdevice_sda_vendor => VMware
blockdevice_sdb_model => Virtual disk
blockdevice_sdb_size => 214748364800
blockdevice_sdb_vendor => VMware
blockdevice_sdc_model => Virtual disk
blockdevice_sdc_size => 32212254720
blockdevice_sdc_vendor => VMware
blockdevice_sr0_model => VMware IDE CDR10
blockdevice_sr0_size => 64784384
blockdevice_sr0_vendor => NECVMWar
blockdevices => fd0,sda,sdb,sdc,sr0
boardmanufacturer => Intel Corporation
boardproductname => 440BX Desktop Reference Platform
boardserialnumber => None
domain => example.com
facterversion => 1.7.6
filesystems => btrfs,ext2,ext3,ext4,msdos,vfat,xfs
fqdn => plin-w1rhns01.example.com
hardwareisa => x86_64
hardwaremodel => x86_64
hostname => plin-w1rhns01
id => root
interfaces => ens192,lo
ipaddress => 172.23.27.50
ipaddress_ens192 => 172.23.27.50
ipaddress_lo => 127.0.0.1
is_virtual => true
kernel => Linux
kernelmajversion => 3.10
kernelrelease => 3.10.0-229.7.2.el7.x86_64
kernelversion => 3.10.0
macaddress => 00:50:56:b0:38:95
macaddress_ens192 => 00:50:56:b0:38:95
manufacturer => VMware, Inc.
memoryfree => 5.30 GB
memoryfree_mb => 5423.56
memorysize => 11.58 GB
memorysize_mb => 11855.77
memorytotal => 11.58 GB
netmask => 255.255.255.240
netmask_ens192 => 255.255.255.240
netmask_lo => 255.0.0.0
network_ens192 => 172.23.27.48
network_lo => 127.0.0.0
operatingsystem => RedHat
operatingsystemmajrelease => 7
operatingsystemrelease => 7.1
osfamily => RedHat
path => /usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/root/bin
physicalprocessorcount => 1
processor0 => Intel(R) Xeon(R) CPU E5-2660 0 @ 2.20GHz
processor1 => Intel(R) Xeon(R) CPU E5-2660 0 @ 2.20GHz
processor2 => Intel(R) Xeon(R) CPU E5-2660 0 @ 2.20GHz
processor3 => Intel(R) Xeon(R) CPU E5-2660 0 @ 2.20GHz
processorcount => 4
productname => VMware Virtual Platform
ps => ps -ef
puppetversion => 3.6.2
rubysitedir => /usr/local/share/ruby/site_ruby/
rubyversion => 2.0.0
selinux => false
serialnumber => VMware-42 30 64 8e 9c 94 23 5f-10 74 90 7c 43 38 71 72
sshecdsakey => thisisakey
sshfp_ecdsa => SSHFP 3 1 fingerprint1
SSHFP 3 2 fingerprint2
sshfp_rsa => SSHFP 1 1 fingerprint3
SSHFP 1 2 fingerprint4
sshrsakey => thersapublickey
swapfree => 0.00 MB
swapfree_mb => 0.00
swapsize => 0.00 MB
swapsize_mb => 0.00
timezone => CDT
type => Other
uniqueid => 17ac321b
system_uptime => {"seconds"=>3663199, "hours"=>1017, "days"=>42, "uptime"=>"42 days"}
uptime => 42 days
uptime_days => 42
uptime_hours => 1017
uptime_seconds => 3663199
uuid => 4230648E-9C94-235F-1074-907C43387172
virtual => vmware
""".strip()


def test_facter():
    fc = Facter(context_wrap(FACTS))
    assert fc.uptime_days == "42"
    assert fc.fqdn == 'plin-w1rhns01.example.com'
    assert fc.uuid == '4230648E-9C94-235F-1074-907C43387172'
    assert hasattr(fc, 'no_test') is False
    assert hasattr(fc, 'swapfree') is True
    assert fc.get('uptime') == "42 days"
    assert fc.get('dummy') is None



"""
OsRelease - file ``/etc/os-release``
====================================

This module provides plugins access to file ``/etc/os-release``.

.. note::

    The /etc/os-release is not exist in RHEL6 and prior versions.

"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.os_release)
class OsRelease(Parser, dict):
    """
    Parses the content of file ``/etc/os-release`` and stores it as a `dict`.

    Sample content of this file::

            NAME="Red Hat Enterprise Linux Server"
            VERSION="7.2 (Maipo)"
            ID="rhel"
            ID_LIKE="fedora"
            VERSION_ID="7.2"
            PRETTY_NAME="Employee SKU"
            ANSI_COLOR="0;31"
            CPE_NAME="cpe:/o:redhat:enterprise_linux:7.2:GA:server"
            HOME_URL="https://www.redhat.com/"
            BUG_REPORT_URL="https://bugzilla.redhat.com/"
            REDHAT_BUGZILLA_PRODUCT="Red Hat Enterprise Linux 7"
            REDHAT_BUGZILLA_PRODUCT_VERSION=7.2
            REDHAT_SUPPORT_PRODUCT="Red Hat Enterprise Linux"
            REDHAT_SUPPORT_PRODUCT_VERSION="7.2"


    Raises:
        SkipComponent: When nothing is parsed.

    Examples:
        >>> type(rls)
        <class 'insights.parsers.os_release.OsRelease'>
        >>> rls.get("VARIANT_ID") is None
        True
        >>> rls.get("VERSION") == "7.2 (Maipo)"
        True
    """
    def parse_content(self, content):
        data = {}
        for line in get_active_lines(content):
            k, _, v = line.partition("=")
            if _ == "=" and k:
                data[k] = v.strip('"\'') if v else None
        if not data:
            raise SkipComponent
        self.update(data)

    @property
    def data(self):
        """
        .. warning::
            Deprecated, it will be removed from 3.7.0
        """
        return self

    @property
    def is_rhel_ai(self):
        """
        Returns:
            bool: True when the system is from RHEL AI image.
        """
        return self.get("VARIANT", None) == "RHEL AI" and self.get("VARIANT_ID", None) == "rhel_ai"



from insights.parsers.dirsrv_logs import DirSrvAccessLog, DirSrvErrorsLog
from insights.tests import context_wrap

import datetime

ACCESS_LOG = '''
    389-Directory/1.2.11.15 B2014.300.2010
    ldap.example.com:636 (/etc/dirsrv/slapd-EXAMPLE-COM)

[27/Apr/2015:13:16:35 -0400] conn=1174478 fd=131 slot=131 connection from 10.20.10.106 to 10.20.62.16
[27/Apr/2015:13:16:35 -0400] conn=1174478 op=-1 fd=131 closed - B1
[27/Apr/2015:13:16:35 -0400] conn=324375 op=606903 SRCH base="cn=users,cn=accounts,dc=example,dc=com" scope=2 filter="(uid=root)" attrs=ALL
[27/Apr/2015:13:16:35 -0400] conn=324375 op=606903 RESULT err=0 tag=101 nentries=1 etime=0
[27/Apr/2015:13:16:35 -0400] conn=324375 op=606904 SRCH base="cn=groups,cn=accounts,dc=example,dc=com" scope=2 filter="(|(member=uid=root,cn=users,cn=accounts,dc=example,dc=com)(uniqueMember=uid=root,cn=users,cn=accounts,dc=example,dc=com)(memberUid=root))" attrs="cn"
[27/Apr/2015:13:16:35 -0400] conn=324375 op=606904 RESULT err=0 tag=101 nentries=8 etime=0
[27/Apr/2015:13:16:40 -0400] conn=1174483 fd=131 slot=131 connection from 10.20.130.21 to 10.20.62.16
[27/Apr/2015:13:16:40 -0400] conn=1174483 op=0 SRCH base="" scope=0 filter="(objectClass=*)" attrs="* altServer namingContexts aci"
[27/Apr/2015:13:16:40 -0400] conn=1174483 op=0 RESULT err=0 tag=101 nentries=1 etime=0
'''.strip()

ERRORS_LOG = '''
    389-Directory/1.2.11.15 B2014.300.2010
    ldap.example.com:7390 (/etc/dirsrv/slapd-PKI-IPA)

[23/Apr/2015:23:12:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
[23/Apr/2015:23:17:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
[23/Apr/2015:23:22:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
[23/Apr/2015:23:27:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
[23/Apr/2015:23:32:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
[23/Apr/2015:23:37:31 -0400] slapi_ldap_bind - Error: could not send startTLS request: error -11 (Connect error) errno 0 (Success)
'''


def test_access_log():
    # Note that this tests only one instance; it does not test iterating
    # through the logs as a normal rule would have to
    ctx = context_wrap(ACCESS_LOG, path='/var/log/dirsrv/slapd-EXAMPLE-COM/access')
    log = DirSrvAccessLog(ctx)
    assert log
    assert len(log.lines) == 12
    connects = log.get('connection from')
    assert len(connects) == 2
    assert '10.20.10.106' in connects[0]['raw_message']
    assert '10.20.130.21' in connects[1]['raw_message']
    tstamp = datetime.datetime(2015, 4, 27, 13, 16, 36)
    assert len(list(log.get_after(tstamp))) == 3
    assert len(list(log.get_after(tstamp, 'connection from'))) == 1


def test_error_log():
    # Note that this tests only one instance; it does not test iterating
    # through the logs as a normal rule would have to
    ctx = context_wrap(ERRORS_LOG, path='/var/log/dirsrv/slapd-PKI-IPA/errors')
    log = DirSrvErrorsLog(ctx)
    assert log
    assert len(log.lines) == 9
    connects = log.get('slapi_ldap_bind')
    assert len(connects) == 6
    assert 'could not send startTLS request' in connects[0]['raw_message']
    tstamp = datetime.datetime(2015, 4, 23, 23, 22, 31)
    assert len(list(log.get_after(tstamp))) == 4



"""
Parsers for watchdog configuration
==================================

WatchDogConf - file ``/etc/watchdog.conf``
------------------------------------------
WatchDogLog - file ``/var/log/watchdog/*.std*``
-----------------------------------------------
"""

from insights import parser, Parser, SkipComponent, LogFileOutput
from insights.specs import Specs
from insights.parsers import split_kv_pairs


@parser(Specs.watchdog_conf)
class WatchDogConf(Parser, dict):
    """
    Parsing the `/etc/watchdog.conf` file. It stores the data in a dict.

    Sample content::

        # The retry-timeout and repair limit are used to handle errors in a
        # more robust manner. Errors must persist for longer than this to
        # action a repair or reboot, and if repair-maximum attempts are
        # made without the test passing a reboot is initiated anyway.

        retry-timeout          = 60
        repair-maximum         = 1
        realtime               = yes
        priority               = 1

    Examples:
        >>> 'retry-timeout' in watchdog_conf_obj
        True
        >>> 'test-timeout' in watchdog_conf_obj
        False
        >>> watchdog_conf_obj.get('retry-timeout')
        '60'
        >>> watchdog_conf_obj.get('realtime')
        'yes'
    """
    def parse_content(self, content):
        self.update(split_kv_pairs(content))
        if not self:
            raise SkipComponent('Not available data')


@parser(Specs.watchdog_logs)
class WatchDogLog(LogFileOutput):
    """
    Class for parsing ``/var/log/watchdog/*.std*`` files.

    Sample Input::

        DEBUG:root:0

        INFO:root:Executing: /usr/bin/sg_persist -n -i -k -d /dev/mapper/mpathb

        DEBUG:root:0   PR generation=0xc2, 4 registered reservation keys follow:
            0xfdfe0001
            0xfdfe0001
            0xfdfe0000
            0xfdfe0000


        DEBUG:root:key fdfe0001 registered with device /dev/mapper/mpathb
        INFO:root:Executing: /usr/bin/sg_turs /dev/mapper/mpathb

        DEBUG:root:0
    """

    time_format = None



"""
buddyinfo - file ``/proc/buddyinfo``
====================================

This parsers the contents of ``/proc/buddyinfo``.

Here is from the mannual page of ``/proc/buddyinfo``:

    /proc/buddyinfo
            This file contains information which is used for
            diagnosing memory fragmentation issues.  Each line starts
            with the identification of the node and the name of the
            zone which together identify a memory region.  This is
            then followed by the count of available chunks of a
            certain order in which these zones are split.  The size in
            bytes of a certain order is given by the formula:

                (2^order) * PAGE_SIZE

            The binary buddy allocator algorithm inside the kernel
            will split one chunk into two chunks of a smaller order
            (thus with half the size) or combine two contiguous chunks
            into one larger chunk of a higher order (thus with double
            the size) to satisfy allocation requests and to counter
            memory fragmentation.  The order matches the column
            number, when starting to count at zero.

            If the memory is heavily fragmented, the counters for
            higher order chunks will be zero and allocation of large
            contiguous areas will fail.

This parser will only store the counter of memory fragmentation.
To get the real size of each column, use ``PAGE_SIZE`` from parser
class:``GetconfPageSize`` for calculation.

For each line in ``/proc/buddyinfo``, it will be parsed and stored in a dict
with the following properties:

* ``node`` (str)     - the Node ID, eg. "0"
* ``zone`` (str)     - the zone name, eg. "Normal"
* ``counter`` (list) - the chuck numbers in order
* ``raw`` (str)      - the raw line

Sample data::

    Node 0, zone      DMA      0      0      0      0      0      0      0      0      1      1      2
    Node 0, zone    DMA32      8     10     12      9      9     11     10     12     12     11    569
    Node 0, zone   Normal    460    485    375   1611   3201   2187   1437    844    487    247   6120
    Node 1, zone   Normal      1      7   1783   2063   1773   3227   2299   1399    729    430   6723

Examples:

    >>> len(buddy)
    4
    >>> mem = buddy[2]
    >>> mem['node']
    '0'
    >>> mem['zone']
    'Normal'
    >>> mem['counter']
    [460, 485, 375, 1611, 3201, 2187, 1437, 844, 487, 247, 6120]
    >>> mem['counter'][0]   # (2^0) * PAGE_SIZE (order 0)
    460
    >>> mem['counter'][10]  # (2^10) * PAGE_SIZE (order 10)
    6120
    >>> mem['raw']
    'Node 0, zone   Normal    460    485    375   1611   3201   2187   1437    844    487    247   6120'
"""

from insights.core import Parser
from insights.core.plugins import parser
from insights.core.exceptions import SkipComponent
from insights.specs import Specs


@parser(Specs.buddyinfo)
class BuddyInfo(Parser, list):
    """Prase the contents of ``/proc/buddyinfo``."""

    def parse_content(self, content):
        for line in content:
            node_str, other_str = line.split(',', 1)
            others = other_str.split()
            self.append({
                'node': node_str.split()[-1],
                'zone': others[1],
                'counter': [int(v) for v in others[2:]],
                'raw': line,
            })

        if len(self) < 1:
            raise SkipComponent



"""
SELinux Policy Management tool
==============================

This module contains the following parsers:

LinuxUserCountMapSelinuxUser - datasource ``users_count_map_selinux_user``
--------------------------------------------------------------------------
"""

from insights.specs import Specs
from insights import JSONParser, parser


@parser(Specs.users_count_map_selinux_user)
class LinuxUserCountMapSelinuxUser(JSONParser):
    """
    Parse the output of the datasource ``users_count_map_selinux_user``.
    It returns a dict by transforming the json format.

    Sample Input::

        {
            "staff_u": 2,
            "guest_u": 4
        }

    Examples:
        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.selinux_users, 'staff_u')
        >>> add_filter(Specs.selinux_users, 'guest_u')
        >>> type(users)
        <class 'insights.parsers.semanage.LinuxUserCountMapSelinuxUser'>
        >>> 'staff_u' in users
        True
        >>> users['staff_u']
        2
    """
    pass



import pytest

from collections import defaultdict

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.specs import Specs
from insights.specs.datasources.lsattr import paths_to_lsattr


def setup_function(func):
    if func is test_lsattr_empty:
        return
    if func is test_lsattr_one_file:
        filters.add_filter(Specs.lsattr_files_or_dirs, ["/etc/default/grub"])
    if func is test_lsattr_more_files:
        filters.add_filter(
            Specs.lsattr_files_or_dirs, ["/etc/default/grub", "/etc/httpd/httpd.conf"]
        )


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)


def test_lsattr_empty():
    with pytest.raises(SkipComponent):
        paths_to_lsattr({})


def test_lsattr_one_file():
    attr_files = paths_to_lsattr({})
    assert attr_files == '/etc/default/grub'


def test_lsattr_more_files():
    attr_files = paths_to_lsattr({})
    assert attr_files == '/etc/default/grub /etc/httpd/httpd.conf'



from insights.parsr import Number


def test_zero():
    assert Number("0") == 0.0


def test_positive_integer():
    assert Number("123") == 123.0


def test_negative_integer():
    assert Number("-123") == -123.0


def test_positive_float():
    assert Number("123.97") == 123.97


def test_negative_float():
    assert Number("-123.97") == -123.97



# -*- coding: UTF-8 -*-
from mock.mock import patch

from insights.client.client import collect
from insights.client.config import InsightsConfig

conf_remove_file = "/tmp/remove.conf"
conf_file_redaction_file = "/tmp/file-redaction.yaml"
conf_file_content_redaction_file = "/tmp/file-content-redaction.yaml"
removed_files = ["/etc/some_file", "/tmp/another_file"]


def collect_args(*insights_config_args, **insights_config_custom_kwargs):
    """
    Instantiates InsightsConfig with a default logging_file argument.
    """
    all_insights_config_kwargs = {
        "logging_file": "/tmp/insights.log",
        "remove_file": conf_remove_file,
        "redaction_file": conf_file_redaction_file,
        "content_redaction_file": conf_file_content_redaction_file,
    }
    all_insights_config_kwargs.update(insights_config_custom_kwargs)
    return InsightsConfig(*insights_config_args, **all_insights_config_kwargs)


def patch_get_rm_conf():
    """
    Mocks InsightsUploadConf.get_rm_conf so it returns a fixed configuration.
    """

    def decorator(old_function):
        patcher = patch("insights.client.client.InsightsUploadConf.get_rm_conf")
        return patcher(old_function)

    return decorator


def patch_core_collector():
    """
    Replaces CoreCollector with a dummy mock.
    """

    def decorator(old_function):
        patcher = patch("insights.client.client.CoreCollector")
        return patcher(old_function)

    return decorator


@patch_core_collector()
@patch_get_rm_conf()
def test_get_rm_conf_file(get_rm_conf, core_collector):
    """
    Load configuration of files removed from collection when collection rules are loaded from a file.
    """
    config = collect_args()
    collect(config)

    get_rm_conf.assert_called_once_with()


@patch_core_collector()
@patch_get_rm_conf()
def test_core_collector_file(get_rm_conf, core_collector):
    """
    Configuration from a file is passed to the CoreCollector along with removed files configuration.
    """
    config = collect_args()
    collect(config)

    rm_conf = get_rm_conf.return_value
    core_collector.return_value.run_collection.assert_called_once_with(rm_conf)
    core_collector.return_value.done.assert_called_once_with()


@patch("insights.client.client.CoreCollector")
@patch_get_rm_conf()
def test_correct_collector_loaded(get_rm_conf, core_collector):
    '''
    Verify that core collection is loaded
    '''
    config = collect_args()
    collect(config)

    core_collector.return_value.run_collection.assert_called()

    core_collector.return_value.run_collection.reset_mock()



"""
Custom datasource for collecting yum updates
============================================
"""

import json
import logging
import time

from functools import cmp_to_key

from insights import datasource, HostContext, SkipComponent
from insights.components.rhel_version import IsRhel7, IsRhel8, IsRhel9
from insights.core.spec_factory import DatasourceProvider
from insights.util.rpm_vercmp import version_compare


def sorted_cmp(it, cmp):
    return sorted(it, key=cmp_to_key(cmp))


class DnfManager:
    """Performs package resolution on dnf based systems"""

    def __init__(self, build_pkgcache=False):
        self.base = dnf.base.Base()
        self.base.conf.cacheonly = not build_pkgcache
        # releasever and basearchs are correctly set after calling load()
        self.releasever = dnf.rpm.detect_releasever("/")
        self.basearch = dnf.rpm.basearch(hawkey.detect_arch())
        self.packages = []
        self.repos = []

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass

    @staticmethod
    def pkg_cmp(a, b):
        if a.name != b.name:
            return -1 if a.name < b.name else 1
        vercmp = rpm.labelCompare((str(a.e), a.v, a.r), (str(b.e), b.v, b.r))
        if vercmp != 0:
            return vercmp
        if a.reponame != b.reponame:
            return -1 if a.reponame < b.reponame else 1
        return 0

    def sorted_pkgs(self, pkgs):
        # if package is installed more than once (e.g. kernel)
        # don't report other installed (i.e. with @System repo) as updates
        return sorted_cmp([pkg for pkg in pkgs if pkg.reponame != "@System"], self.pkg_cmp)

    def load(self):
        logging.disable(logging.WARNING)
        cli = dnf.cli.Cli(self.base)
        cli._read_conf_file()
        self.base.init_plugins()
        self.base.pre_configure_plugins()
        subst = self.base.conf.substitutions
        if subst.get("releasever"):
            self.releasever = subst["releasever"]
        if subst.get("basearch"):
            self.basearch = subst["basearch"]

        self.base.read_all_repos()
        self.base.configure_plugins()

        self.packages = hawkey.Query(hawkey.Sack())
        try:
            if version_compare(dnf.VERSION, "4.7.0") >= 0 and self.base.conf.cacheonly:
                self.base.fill_sack_from_repos_in_cache(load_system_repo=True)
                self.packages = self.base.sack.query()
            elif not self.base.conf.cacheonly:
                self.base.fill_sack()
                self.packages = self.base.sack.query()
        except dnf.exceptions.RepoError:
            # RepoError is raised when cache is empty
            pass

        self.repos = self.base.repos
        logging.disable(logging.NOTSET)

    def installed_packages(self):
        return self.packages.installed().run()

    def updates(self, pkg):
        name = pkg.name
        evr = "{0}:{1}-{2}".format(pkg.epoch, pkg.version, pkg.release)
        arch = pkg.arch
        nevra = "{0}-{1}.{2}".format(name, evr, arch)
        updates_list = []
        for upd in self.packages.filter(name=name, arch=arch, evr__gt=evr):
            updates_list.append(upd)
        return nevra, updates_list

    @staticmethod
    def pkg_nevra(pkg):
        return "{0}-{1}:{2}-{3}.{4}".format(pkg.name, pkg.epoch, pkg.version, pkg.release, pkg.arch)

    @staticmethod
    def pkg_repo(pkg):
        return pkg.reponame

    @staticmethod
    def sorted_advisories(pkg):
        advisories = pkg.get_advisories(hawkey.EQ)
        if advisories:
            advisories_ids = map(lambda advisory: advisory.id, advisories)
            return sorted(advisories_ids)
        return []

    def last_update(self):
        last_ts = 0
        for repo in self.base.repos.iter_enabled():
            repo_ts = repo._repo.getTimestamp()
            if repo_ts > last_ts:
                last_ts = repo_ts
        return last_ts


class YumManager(DnfManager):
    """Performs package resolution on yum based systems"""

    def __init__(self, build_pkgcache=False):
        self.base = yum.YumBase()
        self.base.doGenericSetup(cache=0 if build_pkgcache else 1)
        self.releasever = self.base.conf.yumvar['releasever']
        self.basearch = self.base.conf.yumvar['basearch']
        self.naevr_to_repo = {}
        self.pkg_tup_to_advisories = {}

    @staticmethod
    def pkg_cmp(a_tup, b_tup):
        a_n, a_a, a_e, a_v, a_r = a_tup[0]
        b_n, b_a, b_e, b_v, b_r = b_tup[0]
        a_repoid = a_tup[1]
        b_repoid = b_tup[1]

        if a_n != b_n:
            return -1 if a_n < b_n else 1
        vercmp = rpmUtils.miscutils.compareEVR(
            (
                a_e,
                a_v,
                a_r,
            ),
            (
                b_e,
                b_v,
                b_r,
            ),
        )
        if vercmp != 0:
            return vercmp
        if a_repoid != b_repoid:
            return -1 if a_repoid < b_repoid else 1
        return 0

    def sorted_pkgs(self, pkg_tups):
        # Sorting on top of array of naevr tuple [(n,a,e,v,r), repo_id]
        return sorted_cmp(pkg_tups, self.pkg_cmp)

    def load(self):
        try:
            self.base.doRepoSetup()
            self.base.doSackSetup()
        except yum.Errors.RepoError:
            # RepoError is raised when cache is empty
            pass
        except AttributeError:
            # backwards compatibility, because yum is removing these setup functions in future
            # and moving the setups to be getters (https://github.com/rpm-software-management/yum/blob/master/yum/__init__.py#L1099)
            pass

        try:
            pkgs = self.base.pkgSack.returnPackages()
            for pkg in pkgs:
                naevr = (
                    pkg.name,
                    pkg.arch,
                    pkg.epoch,
                    pkg.version,
                    pkg.release,
                )
                self.naevr_to_repo.setdefault(naevr, set()).add(pkg.repoid)
        except yum.Errors.RepoError:
            # RepoError is raised when cache is empty
            pass

    def installed_packages(self):
        return self.base.rpmdb.returnPackages()

    def updates(self, pkg):
        updates = self.base.upinfo.get_applicable_notices(
            (
                pkg.name,
                pkg.arch,
                pkg.epoch,
                pkg.version,
                pkg.release,
            )
        )
        updates_list = set()
        # updates are returnes in format [((name, arch, epoch, version, release), UpdateNotice)]
        for naevr, update_notice in updates:
            for repoid in self.naevr_to_repo[naevr]:
                # pkg_tup is composed from ((n,a,e,v,r), repoid)
                pkg_tup = (
                    naevr,
                    repoid,
                )
                update_id = update_notice.get_metadata().get("update_id")
                if update_id:
                    self.pkg_tup_to_advisories.setdefault(pkg_tup, set()).add(update_id)
                updates_list.add(pkg_tup)
        return pkg.nevra, updates_list

    @staticmethod
    def pkg_nevra(pkg_tup):
        # tuple of [(n,a,e,v,r), repo_id] -> nevra
        naevr = pkg_tup[0]
        return "{0}-{1}:{2}-{3}.{4}".format(naevr[0], naevr[2], naevr[3], naevr[4], naevr[1])

    @staticmethod
    def pkg_repo(pkg_tup):
        return pkg_tup[1]

    def sorted_advisories(self, pkg_tup):
        advisories = self.pkg_tup_to_advisories.get(pkg_tup, set())
        return sorted(advisories)

    @staticmethod
    def last_update():
        return 0


# Select which manager to use based on the available system libraries.
try:
    import dnf
    import dnf.cli
    import hawkey
    import rpm

    UpdatesManager = DnfManager
except ImportError:
    try:
        import yum
        import rpmUtils.miscutils

        UpdatesManager = YumManager
    except ImportError:
        UpdatesManager = None


@datasource(HostContext, [IsRhel7, IsRhel8, IsRhel9], timeout=0)
def yum_updates(broker):
    """
    This datasource provides a list of available updates on the system.
    It uses the yum python library installed locally, and collects list of
    available package updates, along with advisory info where applicable.

    Sample data returned::

        {
          "releasever": "8",
          "basearch": "x86_64",
          "update_list": {
            "NetworkManager-1:1.22.8-4.el8.x86_64": {
              "available_updates": [
                {
                  "package": "NetworkManager-1:1.22.8-5.el8_2.x86_64",
                  "repository": "rhel-8-for-x86_64-baseos-rpms",
                  "basearch": "x86_64",
                  "releasever": "8",
                  "erratum": "RHSA-2020:3011"
                }
              ]
            }
          },
          "build_pkgcache": false,
          "metadata_time": "2021-01-01T09:39:45Z"
        }

    Returns:
        list: List of available updates
    Raises:
        SkipComponent: Raised when neither dnf nor yum is found
    """
    if UpdatesManager is None:
        raise SkipComponent()

    build_pkgcache = getattr(broker.get('client_config', object), 'build_packagecache', False)

    with UpdatesManager(build_pkgcache=build_pkgcache) as umgr:
        umgr.load()

        response = {
            "releasever": umgr.releasever,
            "basearch": umgr.basearch,
            "update_list": {},
            "build_pkgcache": build_pkgcache,
        }

        for pkg in umgr.installed_packages():
            nevra, updates_list = umgr.updates(pkg)
            if updates_list:
                out_list = []
                updates_list = umgr.sorted_pkgs(updates_list)

                for p in updates_list:
                    pkg = {
                        "package": umgr.pkg_nevra(p),
                        "repository": umgr.pkg_repo(p),
                        "basearch": response["basearch"],
                        "releasever": response["releasever"],
                    }

                    advisories = umgr.sorted_advisories(p)
                    if advisories:
                        for advisory in advisories:
                            pkg_errata = pkg.copy()
                            pkg_errata["erratum"] = advisory
                            out_list.append(pkg_errata)
                    else:
                        out_list.append(pkg)

                response["update_list"][nevra] = {"available_updates": out_list}

        ts = umgr.last_update()
        if ts:
            response["metadata_time"] = time.strftime("%FT%TZ", time.gmtime(ts))

    return DatasourceProvider(
        content=json.dumps(response), relative_path='insights_datasources/yum_updates'
    )



from insights.tests import context_wrap
from insights.parsers.host_vdsm_id import VDSMId

UUID = 'F7D9D983-6233-45C2-A387-9B0C33CB1306'

UUID_CONTENT = """
# VDSM UUID info
#
F7D9D983-6233-45C2-A387-9B0C33CB1306
""".strip()


def test_get_vdsm_id():
    expected = VDSMId(context_wrap(UUID_CONTENT))
    assert UUID == expected.uuid



import pytest

from mock.mock import Mock
from insights.core.exceptions import SkipComponent
from insights.specs.datasources.db2 import LocalSpecs, db2_users, db2_databases_info
from insights.parsers.ps import PsAuxcww
from insights.combiners.ps import Ps
from insights.core import dr
from insights.tests import context_wrap


PS_AUXCWW = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      1821  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
dbp1      1161530  0.1  1.3 2306928 314076 ?      Sl   Apr19   8:42 db2sysc
dbp2      1161531  0.1  1.3 2306928 314076 ?      Sl   Apr19   8:42 db2sysc
root      1161532  0.1  1.3 2306928 314076 ?      Sl   Apr19   8:42 db2sysc
""".strip()

PS_AUXCWW_NG = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      1821  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
""".strip()

DATABASE_LIST_DBP1 = """

 System Database Directory

 Number of entries in the directory = 2

Database 1 entry:

 Database alias                       = TESTD1
 Database name                        = TESTD1
 Local database directory             = /home/dbp
 Database release level               = 15.00
 Comment                              =
 Directory entry type                 = Indirect
 Catalog database partition number    = 0
 Alternate server hostname            =
 Alternate server port number         =

Database 2 entry:

 Database alias                       = TESTD2
 Database name                        = TESTD2
 Local database directory             = /home/dbp
 Database release level               = 15.00
 Comment                              =
 Directory entry type                 = Indirect
 Catalog database partition number    = 0
 Alternate server hostname            =
 Alternate server port number         =

""".strip()


DATABASE_LIST_DBP2 = """

 System Database Directory

 Number of entries in the directory = 1

Database 1 entry:

 Database alias                       = TESTD1
 Database name                        = TESTD1
 Local database directory             = /home/dbp2
 Database release level               = 15.00
 Comment                              =
 Directory entry type                 = Indirect
 Catalog database partition number    = 0
 Alternate server hostname            =
 Alternate server port number         =

""".strip()

DATABASE_LIST_NO_DBP1 = """

 System Database Directory

 Number of entries in the directory = 1

Database 1 entry:

 Database alias                       = TESTD1
 Local database directory             = /home/dbp
 Database release level               = 15.00
 Comment                              =
 Directory entry type                 = Indirect
 Catalog database partition number    = 0
 Alternate server hostname            =
 Alternate server port number         =

""".strip()


DATABASE_LIST_NO_DBP2 = """

 System Database Directory

 Number of entries in the directory = 1

Database 1 entry:

 Database alias                       = TESTD1
 Local database directory             = /home/dbp2
 Database release level               = 15.00
 Comment                              =
 Directory entry type                 = Indirect
 Catalog database partition number    = 0
 Alternate server hostname            =
 Alternate server port number         =

""".strip()


def test_db2_users():
    ps_auxcww = PsAuxcww(context_wrap(PS_AUXCWW))
    ps = Ps(None, None, None, None, ps_auxcww, None)

    broker = dr.Broker()
    broker[Ps] = ps
    result = db2_users(broker)
    assert result == ["dbp1", "dbp2"]


def test_no_db2_users():
    ps_auxcww = PsAuxcww(context_wrap(PS_AUXCWW_NG))
    ps = Ps(None, None, None, None, ps_auxcww, None)

    broker = dr.Broker()
    broker[Ps] = ps
    with pytest.raises(SkipComponent):
        db2_users(broker)


def test_db2_databases_info():
    database_list_dbp1 = Mock()
    database_list_dbp1.content = DATABASE_LIST_DBP1.splitlines()
    database_list_dbp1.cmd = "/usr/sbin/runuser -l  dbp1  -c 'db2 list database directory'"

    database_list_dbp2 = Mock()
    database_list_dbp2.content = DATABASE_LIST_DBP2.splitlines()
    database_list_dbp2.cmd = "/usr/sbin/runuser -l  dbp2  -c 'db2 list database directory'"

    broker = {LocalSpecs.db2_databases: [database_list_dbp1, database_list_dbp2]}
    result = db2_databases_info(broker)
    assert result == [('dbp1', 'TESTD1'), ('dbp1', 'TESTD2'), ('dbp2', 'TESTD1')]


def test_no_db2_databases_info():
    database_list_dbp1 = Mock()
    database_list_dbp1.content = DATABASE_LIST_NO_DBP1.splitlines()
    database_list_dbp1.cmd = "/usr/sbin/runuser -l  dbp1  -c 'db2 list database directory'"

    database_list_dbp2 = Mock()
    database_list_dbp2.content = DATABASE_LIST_NO_DBP2.splitlines()
    database_list_dbp2.cmd = "/usr/sbin/runuser -l  dbp2  -c 'db2 list database directory'"

    broker = {LocalSpecs.db2_databases: [database_list_dbp1, database_list_dbp2]}
    with pytest.raises(SkipComponent):
        db2_databases_info(broker)



"""
OpenVSwitchLogs - files ``ovsdb-server.log`` and ``ovs_vswitchd.log``
=====================================================================

A standard log file reader for logs written by OpenVSwitch.

The logs have a standard format::

    2016-03-08T02:10:01.155Z|01417|vlog|INFO|opened log file /var/log/openvswitch/ovs-vswitchd.log
    2016-03-08T02:20:05.425Z|01418|connmgr|INFO|br0<->unix: 1 flow_mods in the last 0 s (1 adds)
    2016-03-08T02:20:10.160Z|01419|connmgr|INFO|br0<->unix: 1 flow_mods in the last 0 s (1 deletes)
    2016-03-08T11:30:52.206Z|00013|fatal_signal|WARN|terminating with signal 15 (Terminated)

The ``get`` method breaks up log lines on the bar character ('|') into the
following fields:

* **timestamp** - the UTC time stamp
* **sequence** - the sequence number of this message
* **module** - the module in OpenVSwitch that emitted this error
* **level** - the level of error (INFO, WARN, ERROR)
* **message** - the rest of the message.

Each line of the resultant list is a dictionary with those fields.

Examples:

    >>> vswlog = shared[OVSDB_Server_Log]
    >>> 'fatal_signal' in vswlog
    True
    >>> vswlog.get('fatal_signal')
    [{'timestamp': '2016-03-08T11:30:52.206Z', 'sequence': '00013',
      'module': 'fatal_signal', 'level': 'WARN',
      'message': 'terminating with signal 15 (Terminated)',
      'raw_message': '2016-03-08T11:30:52.206Z|00013|fatal_signal|WARN|terminating with signal 15 (Terminated)']
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


class OpenVSwitchLog(LogFileOutput):
    """
    Template class for reading OpenVSwitch logs.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput` for
        more usage information.
    """
    time_format = '%Y-%m-%dT%H:%M:%S'
    _fieldnames = ['timestamp', 'sequence', 'module', 'level', 'message']

    def _parse_line(self, line):
        """
        Parse line into fields.
        """
        fields = line.split('|', 4)  # stop splitting after fourth | found
        line_info = {'raw_message': line}
        if len(fields) == 5:
            line_info.update(dict(zip(self._fieldnames, fields)))
        return line_info

    def __len__(self):
        return len(self.lines)


@parser(Specs.openvswitch_server_log)
class OVSDB_Server_Log(OpenVSwitchLog):
    """
    Parser for the ``ovsdb_server.log`` file, based on the OpenVSwitchLog
    class.
    """
    pass


@parser(Specs.openvswitch_daemon_log)
class OVS_VSwitchd_Log(OpenVSwitchLog):
    """
    Parser for the ``ovs-vswitchd.log`` file, based on the OpenVSwitchLog
    class.
    """
    pass



from insights.parsers import osa_dispatcher_log
from insights.tests import context_wrap

from datetime import datetime

# Note use of r'' here and in tests because of literal '\n' in text of log
OSA_DISPATCHER_LOG = r"""
2015/12/23 04:40:58 -04:00 28307 0.0.0.0: osad/jabber_lib.__init__
2015/12/23 04:40:58 -04:00 28307 0.0.0.0: osad/jabber_lib.setup_connection('Connected to jabber server', u'example.com')
2015/12/23 04:40:58 -04:00 28307 0.0.0.0: osad/osa_dispatcher.fix_connection('Upstream notification server started on port', 1290)
2015/12/23 04:40:58 -04:00 28307 0.0.0.0: osad/jabber_lib.process_forever
2015/12/27 22:48:50 -04:00 28307 0.0.0.0: osad/jabber_lib.main('ERROR', 'Error caught:')
2015/12/27 22:48:50 -04:00 28307 0.0.0.0: osad/jabber_lib.main('ERROR', 'Traceback (most recent call last):\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 121, in main\\n    self.process_forever(c)\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 179, in process_forever\\n    self.process_once(client)\\n  File "/usr/share/rhn/osad/osa_dispatcher.py", line 187, in process_once\\n    client.retrieve_roster()\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 729, in retrieve_roster\\n    stanza = self.get_one_stanza()\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 801, in get_one_stanza\\n    self.process(timeout=tm)\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 1063, in process\\n    self._parser.Parse(data)\\n  File "/usr/lib/python2.6/site-packages/jabber/xmlstream.py", line 269, in unknown_endtag\\n    self.dispatch(self._mini_dom)\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 829, in _orig_dispatch\\n    jabber.Client.dispatch(self, stanza)\\n  File "/usr/lib/python2.6/site-packages/jabber/jabber.py", line 290, in dispatch\\n    else: handler[\'func\'](self,stanza)\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 380, in dispatch\\n    callback(client, stanza)\\n  File "/usr/share/rhn/osad/dispatcher_client.py", line 145, in _message_callback\\n    sig = self._check_signature_from_message(stanza, actions)\\n  File "/usr/share/rhn/osad/jabber_lib.py", line 1325, in _check_signature_from_message\\n    sig = self._check_signature(stanza, actions=actions)\\n  File "/usr/share/rhn/osad/dispatcher_client.py", line 69, in _check_signature\\n    row = lookup_client_by_name(x_client_id)\\n  File "/usr/share/rhn/osad/dispatcher_client.py", line 213, in lookup_client_by_name\\n    raise InvalidClientError(client_name)\\nInvalidClientError: 870d55ffbb949fae\\n')
"""

OSA_DISPATCHER_TRUNC = r"""
2015/12/27 22:48:50 -04:00 28307 0.0.0
"""


def test_get_osa_dispatcher_log():
    log = osa_dispatcher_log.OSADispatcherLog(context_wrap(
        OSA_DISPATCHER_LOG, path='var/log/rhn/osa-dispatcher.log'
    ))

    # Check the fields in a typical full line
    line = log.get('setup_connection')[0]
    assert line['timestamp'] == '2015/12/23 04:40:58 -04:00'
    assert line['pid'] == '28307'
    assert line['client_ip'] == '0.0.0.0'
    assert line['module'] == 'osad'
    assert line['function'] == 'jabber_lib.setup_connection'
    assert line['info'] == "'Connected to jabber server', u'example.com'"

    # Check lines that don't have an info field
    line = log.get('process_forever')[0]
    assert line['info'] is None

    last = log.last()
    assert last['timestamp'] == '2015/12/27 22:48:50 -04:00'
    assert last['pid'] == '28307'
    assert last['client_ip'] == '0.0.0.0'
    assert last['module'] == 'osad'
    assert last['function'] == 'jabber_lib.main'
    # Problems with literal '\n' in line for comparison, so fall back...
    assert last['info'].startswith("""'ERROR', 'Traceback (most recent call last):""")

    # Test get_after functionality with different inputs
    assert len(list(log.get_after(datetime(2015, 12, 27, 22, 48, 40)))) == 2
    assert len(list(log.get_after(datetime(2015, 12, 23, 4, 40, 0), 'connection'))) == 2

    trunc = osa_dispatcher_log.OSADispatcherLog(context_wrap(
        OSA_DISPATCHER_TRUNC, path='var/log/rhn/osa-dispatcher.log'
    ))
    last = trunc.last()
    assert last['raw_message'] == '2015/12/27 22:48:50 -04:00 28307 0.0.0'
    for k in ['timestamp', 'pid', 'client_ip', 'module', 'function', 'info']:
        assert k not in last



"""
Parsers for ``falconctl`` command outputs
=========================================

This module provides the following parsers:

FalconctlBackend - command ``/opt/CrowdStrike/falconctl -g --backend``
----------------------------------------------------------------------

FalconctlRfm - command ``/opt/CrowdStrike/falconctl -g --rfm-state``
--------------------------------------------------------------------

FalconctlAid - command ``/opt/CrowdStrike/falconctl -g --aid``
--------------------------------------------------------------

FalconctlVersion - command ``/opt/CrowdStrike/falconctl -g --version``
----------------------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent, ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.falconctl_backend)
class FalconctlBackend(CommandParser):
    """
    This parser reads the output of ``/opt/CrowdStrike/falconctl -g --backend``,
    return the back-end mode as a string.

    Example output::

        backend is not set.
        or
        backend=auto.

    Examples:
        >>> type(falconctlbackend)
        <class 'insights.parsers.falconctl.FalconctlBackend'>
        >>> falconctlbackend.backend
        'auto'

    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty.")
        self.backend = ""
        if len(content) == 1 and "=" in content[0]:
            self.backend = content[0].split(".")[0].split("=")[-1].strip()
        elif len(content) == 1 and " is not set." in content[0]:
            self.backend = "not set"

        if not self.backend:
            raise ParseException("Invalid content: {0}".format(content))


@parser(Specs.falconctl_rfm)
class FalconctlRfm(CommandParser):
    """
    This parser reads the output of ``/opt/CrowdStrike/falconctl -g --rfm-state``,
    return the Reduced Functionality Mode as boolean.

    Example output::

        rfm-state=false.

    Examples:
        >>> type(falconctlrfm)
        <class 'insights.parsers.falconctl.FalconctlRfm'>
        >>> falconctlrfm.rfm
        False

    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty.")
        self.rfm = False
        state = content[0].split(".")[0].split("=")[-1].strip()
        if state == "true":
            self.rfm = True


@parser(Specs.falconctl_aid)
class FalconctlAid(CommandParser):
    """
    This parser reads the output of ``/opt/CrowdStrike/falconctl -g --aid``,
    return the agent id as a string.

    Example output::

        aid="44e3b7d20b434a2bb2815d9808fa3a8b".

    Examples:
        >>> type(falconctlaid)
        <class 'insights.parsers.falconctl.FalconctlAid'>
        >>> falconctlaid.aid
        '44e3b7d20b434a2bb2815d9808fa3a8b'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty.")

        self.aid = None
        if len(content) == 1 and "=" in content[0]:
            self.aid = content[0].split(".")[0].split("=")[-1].strip('" ')
        elif len(content) == 1 and " is not set." in content[0]:
            self.aid = "not set"

        if not self.aid:
            raise ParseException("Invalid content: {0}".format(content))


@parser(Specs.falconctl_version)
class FalconctlVersion(CommandParser):
    """
    This parser reads the output of ``/opt/CrowdStrike/falconctl -g --version``,
    return the running falcon_sensor version.

    Example output::

        version = 7.14.16703.0

    Examples:
        >>> type(falconctlversion)
        <class 'insights.parsers.falconctl.FalconctlVersion'>
        >>> falconctlversion.version
        '7.14.16703.0'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty.")

        self.version = None
        if "=" in content[0]:
            self.version = content[0].split("=")[-1].strip()

        if not self.version:
            raise ParseException("Invalid content: {0}".format(content))



import pytest
import doctest

from insights.parsers import rhc
from insights.parsers.rhc import RhcConf
from insights.tests import context_wrap
from insights.core.dr import SkipComponent


RHC_CONFIG = """
# rhc global configuration settings

broker = ["wss://connect.cloud.redhat.com:443"]
cert-file = "/etc/pki/consumer/cert.pem"
"key-file" = "/etc/pki/consumer/key.pem"
invalid lines
log-level = "error"
""".strip()

RHC_EMPTY_CONFIG = """
# rhc global configuration settings
""".strip()


def test_doc():
    failed, _ = doctest.testmod(rhc, globs={
        'rhc_conf': RhcConf(context_wrap(RHC_CONFIG))
    })
    assert failed == 0


def test_rhc():
    rhc = RhcConf(context_wrap(RHC_CONFIG))
    assert len(rhc) == 4
    assert rhc['log-level'] == 'error'
    assert rhc['key-file'] == '/etc/pki/consumer/key.pem'
    assert rhc['cert-file'] == "/etc/pki/consumer/cert.pem"
    assert rhc['broker'] == '["wss://connect.cloud.redhat.com:443"]'


def test_exception():
    with pytest.raises(SkipComponent):
        RhcConf(context_wrap(RHC_EMPTY_CONFIG))



"""
Custom datasources for ``lsattr`` command
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.filters import get_filters
from insights.core.plugins import datasource
from insights.specs import Specs


@datasource(HostContext)
def paths_to_lsattr(broker):
    """
    Get the files or directories by the filters from Specs.lsattr_files_or_dirs

    Returns:
        str: A string by joining all the paths by whitespace

    Raises:
        SkipComponent: when there is not any filter
    """
    filters = sorted(get_filters(Specs.lsattr_files_or_dirs))
    if filters:
        return ' '.join(filters)
    raise SkipComponent



"""
ProcLimits - File ``/proc/<PID>/limits``
========================================

Parser for parsing the the ``limits`` file under special ``/proc/<PID>``
directory.

"""
from insights.core import LegacyItemAccess, Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import parse_fixed_table
from insights.specs import Specs

HEADER_SUBSTITUTE = [('Soft Limit', 'Soft_Limit'), ('Hard Limit', 'Hard_Limit')]


class Limits(LegacyItemAccess):
    """
    An object representing a line in the ``/proc/limits``.  Each entry contains
    below fixed attributes:

    Attributes:
        hard_limit(str): Hard limit
        soft_limit(str): Soft limit
        units(str): Unit of the limit value
    """

    def __init__(self, data={}):
        self.data = data
        for k, v in data.items():
            setattr(self, k, v)

    def items(self):
        """
        To keep backward compatibility and let it can be iterated as a
        dictionary.
        """
        for k, v in self.data.items():
            yield k, v


class ProcLimits(Parser):
    """
    Base class for parsing the ``limits`` file under special ``/proc/<PID>``
    directory into a list of dictionaries by using the
    :py:func:`insights.parsers.parse_fixed_table` function.

    Each line is a dictionary of fields, named according to their definitions
    in ``Limit``.

    This class provides the '__len__' and '__iter__' methods to allow it to
    be used as a list to iterate over the parsed dictionaries.

    Each of the resource provided by this file will be set as an attribute.
    The attribute name is the resource name got from the ``Limit`` column, which
    is converted to lowercase and joined the words with underline '_'.  If not
    sure about whether an attribute is exist or not, check it via the
    '__contains__' method before fetching it.
    The attribute value is set to a :class:`Limits` which wraps the
    corresponding ``hard_limit``, ``soft_limit`` and ``units``.

    Typical content looks like::

        Limit                     Soft Limit           Hard Limit           Units
        Max cpu time              unlimited            unlimited            seconds
        Max file size             unlimited            unlimited            bytes
        Max data size             unlimited            unlimited            bytes
        Max stack size            10485760             unlimited            bytes
        Max core file size        0                    unlimited            bytes
        Max resident set          unlimited            unlimited            bytes
        Max processes             9                    99                   processes
        Max open files            1024                 4096                 files
        Max locked memory         65536                65536                bytes
        Max address space         unlimited            unlimited            bytes
        Max file locks            unlimited            unlimited            locks
        Max pending signals       15211                15211                signals
        Max msgqueue size         819200               819200               bytes
        Max nice priority         0                    0
        Max realtime priority     0                    0
        Max realtime timeout      unlimited            unlimited            us

    Examples:
        >>> len(proc_limits)
        16
        >>> proc_limits.max_processes.hard_limit
        '99'
        >>> proc_limits.max_processes.soft_limit
        '9'
        >>> 'max_cpu_time' in proc_limits
        True
        >>> proc_limits.max_cpu_time.soft_limit
        'unlimited'
        >>> proc_limits.max_cpu_time.units
        'seconds'

    Raises:
        insights.core.exceptions.ParseException: if the ``limits`` file is empty or
            doesn't exist.
    """

    def __contains__(self, key):
        return any(key == row['Limit'] for row in self.data)

    def __len__(self):
        return len(self.data)

    def __iter__(self):
        for row in self.data:
            yield row

    def parse_content(self, content):
        if len(content) == 0 or 'No such file or directory' in content[0]:
            raise ParseException("Error: ", content[0] if content else 'empty file')
        self.data = parse_fixed_table(content, header_substitute=HEADER_SUBSTITUTE)
        for row in self.data:
            row['Limit'] = row['Limit'].lower().replace(' ', '_')
            setattr(self, row['Limit'],
                    Limits({'hard_limit': row['Hard_Limit'],
                            'soft_limit': row['Soft_Limit'],
                            'units': row['Units']}))


@parser(Specs.httpd_limits)
class HttpdLimits(ProcLimits):
    """
    Class for parsing the ``limits`` file of the ``httpd`` process.
    """
    pass


@parser(Specs.mysqld_limits)
class MysqldLimits(ProcLimits):
    """
    Class for parsing the ``limits`` file of the ``mysqld`` process.
    """
    pass


@parser(Specs.ovs_vswitchd_limits)
class OvsVswitchdLimits(ProcLimits):
    """
    Class for parsing the ``limits`` file of the ``ovs-vswitchd`` process.
    """
    pass



"""
docker_host_machineid_parser - File ``/etc/redhat-access-insights/machine-id``
==============================================================================

This is a fairly simple function to read the Insights machine ID.

Because of the way this parser is used in the docker rules, this returns a
one-element dictionary with the machine ID referred to by the key
``host_system_id``.

Examples:

    >>> docker_info = {}
    >>> if docker_host_machineid_parser in shared:
    ...     docker_info.extend(shared[docker_host_machineid_parser])
    >>> docker_info
    { 'host_system_id': '123456789' }

"""

from .. import parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.docker_host_machine_id)
def docker_host_machineid_parser(context):
    """
    Returns:
        (dict): Return the Insights machine ID in a dict with the key
        'host_system_id'.
    """
    deprecated(docker_host_machineid_parser, "This parser is deprecated", "3.7.0")

    content = list(context.content)[0]
    return {"host_system_id": content}



import doctest
from insights.parsers import neutron_metadata_agent_conf
from insights.parsers.neutron_metadata_agent_conf import NeutronMetadataAgentIni
from insights.tests import context_wrap


METADATA_AGENT_INI = """
[DEFAULT]
# Show debugging output in log (sets DEBUG log level output)
# debug = True
debug = False

# The Neutron user information for accessing the Neutron API.
auth_url = http://localhost:35357/v2.0
# Turn off verification of the certificate for ssl
# auth_insecure = False
auth_insecure = False
# Certificate Authority public key (CA cert) file for ssl
# auth_ca_cert =
admin_tenant_name = service
admin_user = neutron
admin_password = *********

# Network service endpoint type to pull from the keystone catalog
# endpoint_type = adminURL

# IP address used by Nova metadata server
# nova_metadata_ip = 127.0.0.1
nova_metadata_ip = 127.0.0.1

# TCP Port used by Nova metadata server
# nova_metadata_port = 8775
nova_metadata_port = 8775

# Which protocol to use for requests to Nova metadata server, http or https
# nova_metadata_protocol = http
nova_metadata_protocol = http

# Whether insecure SSL connection should be accepted for Nova metadata server
# requests
# nova_metadata_insecure = False

# Client certificate for nova api, needed when nova api requires client
# certificates
# nova_client_cert =

# Private key for nova client certificate
# nova_client_priv_key =

# When proxying metadata requests, Neutron signs the Instance-ID header with a
# shared secret to prevent spoofing.  You may select any string for a secret,
# but it must match here and in the configuration used by the Nova Metadata
# Server. NOTE: Nova uses the same config key, but in [neutron] section.
# metadata_proxy_shared_secret =
metadata_proxy_shared_secret =*********

# Location of Metadata Proxy UNIX domain socket
# metadata_proxy_socket = $state_path/metadata_proxy

# Metadata Proxy UNIX domain socket mode, 4 values allowed:
# 'deduce': deduce mode from metadata_proxy_user/group values,
# 'user': set metadata proxy socket mode to 0o644, to use when
# metadata_proxy_user is agent effective user or root,
# 'group': set metadata proxy socket mode to 0o664, to use when
# metadata_proxy_group is agent effective group,
# 'all': set metadata proxy socket mode to 0o666, to use otherwise.
# metadata_proxy_socket_mode = deduce

# Number of separate worker processes for metadata server. Defaults to
# half the number of CPU cores
# metadata_workers =
metadata_workers =0

# Number of backlog requests to configure the metadata server socket with
# metadata_backlog = 4096
metadata_backlog = 4096

# URL to connect to the cache backend.
# default_ttl=0 parameter will cause cache entries to never expire.
# Otherwise default_ttl specifies time in seconds a cache entry is valid for.
# No cache is used in case no value is passed.
# cache_url = memory://?default_ttl=5
cache_url = memory://?default_ttl=5

[AGENT]
# Log agent heartbeats from this Metadata agent
log_agent_heartbeats = False
""".strip()


def test_neutron_metadata_agent_ini():
    nmda_ini = NeutronMetadataAgentIni(context_wrap(METADATA_AGENT_INI))
    assert nmda_ini.has_option('AGENT', 'log_agent_heartbeats')
    assert nmda_ini.get("DEFAULT", "auth_url") == 'http://localhost:35357/v2.0'
    assert nmda_ini.getint("DEFAULT", "metadata_backlog") == 4096


def test_doc():
    env = {'metadata_agent_ini': NeutronMetadataAgentIni(context_wrap(METADATA_AGENT_INI, path='/etc/neutron/metadata_agent.ini'))}
    failed, total = doctest.testmod(neutron_metadata_agent_conf, globs=env)
    assert failed == 0



"""
cron_log - File ``/var/log/cron``
=================================
"""

from .. import Syslog, parser
from insights.specs import Specs


@parser(Specs.cron_log)
class CronLog(Syslog):
    """
    Read the ``/var/log/cron`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.Syslog` for more
        details.

    Sample input::

        Feb 19 03:49:01 smtcfc0247 CROND[899752]: (root) CMD (/etc/run /etc/cron/every15Minutes /var/venus/logs/cron/every15Minutes 5 4.4 >/dev/null 2>&1)
        Feb 19 03:49:01 smtcfc0247 CROND[899751]: (root) CMD (/etc/run /etc/cron/everyHour /var/venus/logs/cron/everyHour 5 65.8 >/dev/null 2>&1)
        Feb 19 03:49:01 smtcfc0247 CROND[899753]: (root) CMD (/etc/run /etc/cron/every5Minutes /var/venus/logs/cron/every5Minutes 5 43.0 >/dev/null 2>&1)
        Feb 19 03:49:01 smtcfc0247 CROND[908273]: (root) CMD (run-parts /etc/cron.hourly)

    Examples:
        >>> run_lines = cron_log.get('/etc/run')
        >>> len(run_lines)
        3
        >>> run_lines[0]['message'] = '(root) CMD (/etc/run /etc/cron/every15Minutes /var/venus/logs/cron/every15Minutes 5 4.4 >/dev/null 2>&1)'
        >>> run_lines[0]['procname'] = 'CROND[899752]'
        >>> run_lines[0]['timestamp'] = 'Feb 19 03:49:01'
    """
    pass



import pytest

from insights.core.exceptions import ParseException
from insights.parsers import xfs_info
from insights.tests import context_wrap


def test_example_xfs_info():
    xfs_obj = xfs_info.XFSInfo(context_wrap("""
meta-data=/dev/sda      isize=256    agcount=32, agsize=16777184 blks
         =              sectsz=512   attr=2
data     =              bsize=4096   blocks=536869888, imaxpct=5
         =              sunit=32     swidth=128 blks
naming   =version 2     bsize=4096
log      =internal      bsize=4096   blocks=32768, version=2
         =              sectsz=512   sunit=32 blks, lazy-count=1
realtime =none          extsz=524288 blocks=0, rtextents=0
    """, path='sos_commands/xfs/xfs_info_.'))
    xfs = xfs_obj.xfs_info

    # Section checks
    assert 'meta-data' in xfs
    assert 'data' in xfs
    assert 'naming' in xfs
    assert 'log' in xfs
    assert 'realtime' in xfs

    # Section specifier checks
    assert xfs['meta-data']['specifier'] == '/dev/sda'
    assert xfs['naming']['specifier'] == 'version'
    assert xfs['naming']['specifier_value'] == '2'
    assert xfs['log']['specifier'] == 'internal'
    assert xfs['realtime']['specifier'] == 'none'

    # Data checks
    assert xfs['meta-data']['isize'] == 256
    assert xfs['meta-data']['agcount'] == 32
    assert xfs['meta-data']['agsize'] == '16777184 blks'
    assert xfs['meta-data']['sectsz'] == 512
    assert xfs['meta-data']['attr'] == 2

    assert xfs['data']['bsize'] == 4096
    assert xfs['data']['blocks'] == 536869888
    assert xfs['data']['imaxpct'] == 5
    assert xfs['data']['sunit'] == 32
    assert xfs['data']['swidth'] == '128 blks'

    assert xfs['naming']['bsize'] == 4096

    assert xfs['log']['bsize'] == 4096
    assert xfs['log']['blocks'] == 32768
    assert xfs['log']['version'] == 2
    assert xfs['log']['sectsz'] == 512
    assert xfs['log']['sunit'] == '32 blks'
    assert xfs['log']['lazy-count'] == 1

    assert xfs['realtime']['extsz'] == 524288
    assert xfs['realtime']['blocks'] == 0
    assert xfs['realtime']['rtextents'] == 0

    # Calculated information checks
    assert xfs_obj.device == '/dev/sda'
    assert xfs_obj.data_size == 536869888 * 4096
    assert xfs_obj.log_size == 32768 * 4096
    assert xfs_obj.mount == '/'


def test_root_xfs_info():
    xfs_obj = xfs_info.XFSInfo(context_wrap("""
meta-data=/dev/mapper/vgSys-lvRoot isize=256    agcount=4, agsize=320000 blks
         =                       sectsz=512   attr=2, projid32bit=1
         =                       crc=0        finobt=0
data     =                       bsize=4096   blocks=1280000, imaxpct=25
         =                       sunit=0      swidth=0 blks
naming   =version 2              bsize=4096   ascii-ci=0 ftype=0
log      =internal               bsize=4096   blocks=2560, version=2
         =                       sectsz=512   sunit=0 blks, lazy-count=1
realtime =none                   extsz=4096   blocks=0, rtextents=0
    """, path='sos_commands/xfs/xfs_info_.'))
    xfs = xfs_obj.xfs_info

    # Section checks
    assert 'meta-data' in xfs
    assert 'data' in xfs
    assert 'naming' in xfs
    assert 'log' in xfs
    assert 'realtime' in xfs

    # Section specifier checks
    assert xfs['meta-data']['specifier'] == '/dev/mapper/vgSys-lvRoot'
    assert xfs['naming']['specifier'] == 'version'
    assert xfs['naming']['specifier_value'] == '2'
    assert xfs['log']['specifier'] == 'internal'
    assert xfs['realtime']['specifier'] == 'none'

    # Data checks
    assert xfs['meta-data']['isize'] == 256
    assert xfs['meta-data']['agcount'] == 4
    assert xfs['meta-data']['agsize'] == '320000 blks'
    assert xfs['meta-data']['sectsz'] == 512
    assert xfs['meta-data']['attr'] == 2
    assert xfs['meta-data']['projid32bit'] == 1
    assert xfs['meta-data']['crc'] == 0
    assert xfs['meta-data']['finobt'] == 0

    assert xfs['data']['bsize'] == 4096
    assert xfs['data']['blocks'] == 1280000
    assert xfs['data']['imaxpct'] == 25
    assert xfs['data']['sunit'] == 0
    assert xfs['data']['swidth'] == '0 blks'

    assert xfs['naming']['bsize'] == 4096
    assert xfs['naming']['ascii-ci'] == 0
    assert xfs['naming']['ftype'] == 0

    assert xfs['log']['bsize'] == 4096
    assert xfs['log']['blocks'] == 2560
    assert xfs['log']['version'] == 2
    assert xfs['log']['sectsz'] == 512
    assert xfs['log']['sunit'] == '0 blks'
    assert xfs['log']['lazy-count'] == 1

    assert xfs['realtime']['extsz'] == 4096
    assert xfs['realtime']['blocks'] == 0
    assert xfs['realtime']['rtextents'] == 0

    # Calculated information checks
    assert xfs_obj.device == '/dev/mapper/vgSys-lvRoot'
    assert xfs_obj.data_size == 1280000 * 4096
    assert xfs_obj.log_size == 2560 * 4096
    assert xfs_obj.mount == '/'

    assert str(xfs_obj) == 'xfs_info of /dev/mapper/vgSys-lvRoot'


def test_ext_log_xfs_info():
    xfs_obj = xfs_info.XFSInfo(context_wrap("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
         =                       sectsz=512   attr=2, projid32bit=1
         =                       crc=0        finobt=0
data     =                       bsize=4096   blocks=262144, imaxpct=25
         =                       sunit=0      swidth=0 blks
naming   =version 2              bsize=4096   ascii-ci=0 ftype=0
log      =log.xfs.image          bsize=4096   blocks=25600, version=2
         =                       sectsz=512   sunit=0 blks, lazy-count=1
realtime =none                   extsz=4096   blocks=0, rtextents=0
    """))
    xfs = xfs_obj.xfs_info

    # Just test the few things that might be different
    assert xfs['meta-data']['specifier'] == 'data.xfs.image'
    assert xfs['log']['specifier'] == 'log.xfs.image'

    # Calculated information checks
    assert xfs_obj.data_size == 262144 * 4096
    assert xfs_obj.log_size == 25600 * 4096


def test_bad_xfs_info():
    for input_data, exception in [  # input data, expected exception
        ("xfs_info: bad command or file name",
            "No 'meta-data' section found"),
        ("meta-data=         isize=256    agcount=4, agsize=65536 blks",
            "Device specifier not found in meta-data"),
        ("meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks",
            "No 'data' section found"),
        ("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
data     =                       sunit=0      swidth=0 blks
         """, "'blocks' not defined in data section"),
        ("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
data     =                       sunit=0      blocks=262144
         """, "'bsize' not defined in data section"),
        ("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
data     =                       bsize=4096   blocks=262144, imaxpct=25
         """, "No 'log' section found"),
        ("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
data     =                       bsize=4096   blocks=262144, imaxpct=25
log      =log.xfs.image          bsize=4096   sunit=0 blks, lazy-count=1
         """, "'blocks' not defined in log section"),
        ("""
meta-data=data.xfs.image         isize=256    agcount=4, agsize=65536 blks
data     =                       bsize=4096   blocks=262144, imaxpct=25
log      =log.xfs.image          sectsz=512   blocks=25600, version=2
         """, "'bsize' not defined in log section"),
    ]:
        with pytest.raises(ParseException) as e_info:
            xfs_info.XFSInfo(context_wrap(input_data))
        assert exception in str(e_info.value)



import doctest
from insights.parsers.installed_product_ids import InstalledProductIDs
from insights.parsers import installed_product_ids
from insights.tests import context_wrap


COMMAND_OUTPUT = """
Product Certificate
path: /etc/pki/product-default/69.pem
id: 69
Product Certificate
path: /etc/pki/product/69.pem
id: 69
"""

COMMAND_OUTPUT2 = """
    Product Certificate
    path: /etc/pki/product-default/69.pem
    id: 69
    brand_type:
    brand_name:
    tags: rhel-7,rhel-7-server
    Product Certificate
    path: /etc/pki/product/479.pem
    id: 479
    tags: rhel-8,rhel-8-x86_64
    brand_type:
    brand_name:
"""

COMMAND_SOME_CERT_PART_DATA_OUTPUT3 = """
Product Certificate
path: /etc/pki/product-default/69.pem
id: 69
brand_type:
brand_name:
tags: rhel-7,rhel-7-server
Product Certificate
path: /etc/pki/product/479.pem
id: 479
tags: rhel-8,rhel-8-x86_64
brand_type:
brand_name:
Product Certificate
"""

NONE_FOUND = """
find: '/etc/pki/product-default/': No such file or directory
find: '/etc/pki/product/': No such file or directory
"""

BAD_FILE = """
Unable to read certificate file '/etc/pki/product/some_file.pem': Error loading certificate
"""


def test_installed_product_ids():
    results = InstalledProductIDs(context_wrap(COMMAND_OUTPUT))
    assert results is not None
    assert results.ids == set(['69', '69'])
    assert results.product_certs[0]['path'] == '/etc/pki/product-default/69.pem'
    assert results.product_certs[0]['id'] == '69'
    assert results.product_certs[1]['path'] == '/etc/pki/product/69.pem'
    assert results.product_certs[1]['id'] == '69'

    results = InstalledProductIDs(context_wrap(COMMAND_OUTPUT2, strip=False))
    assert results.product_certs[0]['path'] == '/etc/pki/product-default/69.pem'
    assert results.product_certs[0]['id'] == '69'
    assert results.product_certs[0]['brand_type'] == ''
    assert results.product_certs[0]['brand_name'] == ''
    assert results.product_certs[0]['tags'] == 'rhel-7,rhel-7-server'
    assert results.product_certs[1]['path'] == '/etc/pki/product/479.pem'
    assert results.product_certs[1]['id'] == '479'
    assert results.product_certs[1]['brand_type'] == ''
    assert results.product_certs[1]['brand_name'] == ''
    assert results.product_certs[1]['tags'] == 'rhel-8,rhel-8-x86_64'
    assert len(results.product_certs) == 2
    assert results.ids == set(['69', '479'])

    results = InstalledProductIDs(context_wrap(COMMAND_SOME_CERT_PART_DATA_OUTPUT3))
    assert len(results.product_certs) == 2

    results = InstalledProductIDs(context_wrap(NONE_FOUND))
    assert results is not None
    assert results.ids == set([])

    results = InstalledProductIDs(context_wrap(BAD_FILE))
    assert results is not None
    assert results.ids == set([])


def test_installed_product_ids_docs():
    env = {
        'products': InstalledProductIDs(context_wrap(COMMAND_OUTPUT)),
    }
    failed, _ = doctest.testmod(installed_product_ids, globs=env)
    assert failed == 0
